from NikGapps.config.NikGappsConfig import NikGappsConfig
from NikGapps.helper.overlay.Overlay import Overlay


class NikGappsOverlays:

    @staticmethod
    def get_overlay(android_version):
        overlay_list = []
        config_obj = NikGappsConfig(android_version=android_version)
        for packages in config_obj.package_manager.get_packages("all"):
            for pkg in packages.package_list:
                for overlay in pkg.overlay_list:
                    overlay: Overlay
                    overlay_list.append(overlay)
        return overlay_list



import os
import yaml

from niklibrary.helper.Statics import Statics


class ApkMetaInfo:
    def __init__(self, apk_file_name, android_version):
        self.apkFileName = apk_file_name
        self.minSdkVersion = Statics.get_android_sdk(android_version)
        self.targetSdkVersion = self.minSdkVersion
        self.versionCode = self.minSdkVersion
        self.versionName = android_version

    def to_dict(self):
        return {
            '!!brut.androlib.meta.MetaInfo': None,
            'apkFileName': self.apkFileName,
            'compressionType': False,
            'doNotCompress': ['resources.arsc'],
            'isFrameworkApk': False,
            'packageInfo': {
                'forcedPackageId': '127',
                'renameManifestPackage': None,
            },
            'sdkInfo': {
                'minSdkVersion': self.minSdkVersion,
                'targetSdkVersion': self.targetSdkVersion,
            },
            'sharedLibrary': False,
            'sparseResources': False,
            'unknownFiles': {},
            'usesFramework': {
                'ids': [1],
                'tag': None,
            },
            'version': '1.0.0',
            'versionInfo': {
                'versionCode': self.versionCode,
                'versionName': self.versionName,
            }
        }

    def write(self, directory, filename):
        if not os.path.exists(directory):
            os.makedirs(directory)
        with open(os.path.join(directory, filename), 'w') as file:
            yaml.dump(self.to_dict(), file)




class ConfigObj:

    def __init__(self, key, value, configdesc=None):
        self.key = key
        self.value = value
        self.description = None
        if configdesc is not None:
            self.description = configdesc

    def get_string(self):
        config_str = ""
        if self.description is not None:
            config_str = self.description + "\n"
        config_str += self.key + "=" + str(self.value) + "\n\n"
        return config_str




import xml.etree.ElementTree as eT


class Bool:
    def __init__(self, name, value, comment=None):
        self.name = name
        self.value = str(value).lower()
        self.comment = comment

    def to_xml(self):
        elem = eT.Element('bool', name=self.name)
        elem.text = self.value
        return elem, eT.Comment(self.comment) if self.comment else None



import xml.etree.ElementTree as eT


class StringArray:
    def __init__(self, name, values, comment=None, translatable=False):
        self.name = name
        self.values = values
        self.translatable = str(translatable).lower()
        self.comment = comment

    def to_xml(self):
        elem = eT.Element('string-array', name=self.name, translatable=self.translatable)
        for value in self.values:
            item = eT.SubElement(elem, 'item')
            item.text = value
        return elem, eT.Comment(self.comment) if self.comment else None



from .Package import Package


class AppSet:

    def __init__(self, name, gapps_list=None):
        self.title = name
        if gapps_list is None:
            self.package_list = []
        else:
            self.package_list = gapps_list
        self.config_note = ""

    def add_package(self, pkg: Package):
        self.package_list.append(pkg)



import os
import time
from shutil import copyfile

from niklibrary.git.Git import Git
import git.exc
from niklibrary.git.GitStatics import GitStatics

from NikGapps.helper.Assets import Assets


class TestGit(Git):

    def git_push(self, commit_message=None, push_untracked_files=False, debug=False, rebase=True, pull_first=True, max_retries=3):
        if not self.enable_push:
            print("Git push is disabled, skipping push!")
            return False
        try:
            if pull_first:
                self.git_pull(rebase=rebase, max_retries=max_retries)

            if self.repo.is_dirty(untracked_files=True):
                self.repo.git.add(update=True)  # Update tracked files
                if push_untracked_files:
                    self.repo.git.add(all=True)  # Add all untracked files

                commit_message = commit_message or "Auto Commit"
                self.repo.index.commit(commit_message)
            return self.git_force_push(debug)

        except git.GitCommandError as e:
            print(f"Git command error: {e}")
        except Exception as e:
            print(f"An error occurred: {e}")
        return False

    def git_pull(self, rebase=True, max_retries=3):
        origin = self.repo.remote(name='origin')
        origin.fetch()
        for retry in range(max_retries):
            try:
                if rebase:
                    self.repo.git.pull('--rebase')
                else:
                    self.repo.git.pull()
                break
            except git.GitCommandError as e:
                print(f"Error during pull/rebase: {e}, retrying...")
                time.sleep(10)  # Wait 10 seconds before retrying
        else:
            print("Max retries reached, pull/rebase failed.")
            return False
        return True

    def git_force_push(self, debug=False):
        origin = self.repo.remote(name='origin')
        push_info = origin.push(force=True)
        for info in push_info:
            if "rejected" in info.summary:
                print(info.summary)
                return False
            else:
                print("Pushed to origin.")
                if debug:
                    print(f"Pushed {info.summary}")
        return True

    def update_changelog(self):
        source_file = Assets.changelog
        dest_file = GitStatics.website_repo_dir + os.path.sep + "_data" + os.path.sep + "changelogs.yaml"
        copyfile(source_file, dest_file)
        if self.due_changes():
            print("Updating the changelog to the website")
            self.git_push("Update Changelog")
        else:
            print("There is no changelog to update!")



import json
from pathlib import Path

from NikGapps.build.NikGappsManager import NikGappsManager
from NikGapps.helper import Config
from NikGapps.helper.AppSet import AppSet
from NikGapps.helper.Assets import Assets
from NikGapps.helper.ConfigObj import ConfigObj
from niklibrary.helper.F import F
from NikGapps.helper.Package import Package
from niklibrary.helper.Statics import Statics
from niklibrary.git.GitOp import GitOp
from niklibrary.web.Upload import Upload
from niklibrary.web.Requests import Requests


class NikGappsConfig:
    def __init__(self, android_version=None, config_path=None, raw_config=None,
                 config_name=None,
                 use_zip_config=0):
        self.is_elite = False
        self.creator = "Nikhil Menghani"
        self.package_manager = NikGappsManager(android_version)
        self.android_version = android_version
        self.arch = "arm64"
        self.config_version = Assets.config_versions[str(self.android_version)]
        self.config_path = config_path
        self.use_zip_config = use_zip_config
        self.elite_folder = None
        if config_path is not None:
            self.raw_config = "".join(F.read_string_file(config_path))
            self.config_name = str(Path(self.config_path).name).split(".config")[0]
            if str(self.config_path).__contains__(Statics.dir_sep + "elite" + Statics.dir_sep):
                self.is_elite = True
                self.elite_folder = str(self.config_path).split(Statics.dir_sep)[-2]
                self.creator = Requests.get_folder_access(self.elite_folder)
        else:
            self.raw_config = raw_config if raw_config is not None else self.build_default_nikgapps_config()
            self.config_name = "nikgapps" if config_name is None else config_name
        self.config_dict = self.build_config_dict()
        self.config_objects = self.build_config_objects(self.config_dict)
        self.config_package_list = self.get_config_packages()
        self.debloater_list = self.get_debloater_list()

    def build_config_dict(self):
        lines = {}
        for line in self.raw_config.splitlines():
            if line.__eq__('') or line.__eq__('\n') or line.startswith('#') \
                    or line.startswith("File Not Found") \
                    or line.startswith("UseZipConfig=") \
                    or not line.__contains__("="):
                continue
            lines[line.split('=')[0]] = line.split('=')[1].replace('\n', '')
        return lines

    def build_config_objects(self, config_dict=None):
        config_list = []
        default_mode, install_mode = "default", "install"
        enabled_mode, disabled_mode, use_zip_config = 1, 0, self.use_zip_config
        predefined_configs = {
            "AndroidVersion": self.android_version,
            "Version": self.config_version,
            "LogDirectory": {
                "value": default_mode,
                "description": """# set this to the directory you want to copy the logs to.
# for e.g. LogDirectory="/system/etc" will install the logs to /system/etc/nikgapps_logs directory
# by default it will install it to /sdcard/NikGapps/nikgapps_logs"""
            },
            "InstallPartition": {
                "value": default_mode,
                "description": "# set to /system, /product or /system_ext if you want to force the installation to aforementioned locations"
            },
            "Mode": {
                "value": install_mode,
                "description": "# set to uninstall if you want to uninstall any google app, also set the value of google app below to -1"
            },
            "WipeDalvikCache": {
                "value": enabled_mode,
                "description": "# set WipeDalvikCache=0 if you don't want the installer to wipe dalvik/cache after installing the gapps"
            },
            "WipeRuntimePermissions": {
                "value": disabled_mode,
                "description": "# set WipeRuntimePermissions=1 if you want to wipe runtime permissions"
            },
            "ExecuteBackupRestore": {
                "value": enabled_mode,
                "description": "# Addon.d config, set it to 0 to skip the automatic backup/restore while flashing the rom"
            },
            "UseZipConfig": {
                "value": use_zip_config,
                "description": "# if you want to force the installer to use the config from gapps zip file, set below to 1"
            },
            "OverwriteWithZipConfig": {
                "value": disabled_mode,
                "description": "# if you want to overwrite the config located in /sdcard/NikGapps with gapps zip file, set below to 1. Applicable to decrypted storage only"
            },
            "GmsOptimization": {
                "value": disabled_mode,
                "description": "# set this to 1 if you want to enable gms optimization, careful while doing it, you may experience issues like delayed notification with some Roms"
            },
            "GenerateLogs": {
                "value": enabled_mode,
                "description": "# set this to 0 if you want to skip generating nikgapps logs, if you run into issues, enable it and flash the zip again to get the logs"
            }
        }
        if config_dict is not None:
            cutoff_index = list(config_dict.keys()).index("Core") if "Core" in config_dict else len(config_dict)
            for key, value in list(config_dict.items())[:cutoff_index]:
                if key in predefined_configs:
                    continue
                config_list.append(ConfigObj(key, value))
        config_list.extend(
            ConfigObj(key, config_info["value"], config_info.get("description")) if isinstance(config_info, dict)
            else ConfigObj(key, config_info)
            for key, config_info in predefined_configs.items()
        )
        return config_list

    def build_default_nikgapps_config(self):
        nikgapps_config_lines = "# NikGapps configuration file\n\n"
        nikgapps_config_lines += "# If you are not sure about the config, " \
                                 "just skip making changes to it or comment it by adding # before it\n"
        nikgapps_config_lines += "# visit https://nikgapps.com/misc/2022/02/22/" \
                                 "NikGapps-Config.html to read everything about nikgapps\n\n"

        for config_obj in self.build_config_objects():
            nikgapps_config_lines += config_obj.get_string()

        nikgapps_config_lines += "# Following are the packages with default configuration\n"

        for app_set in self.package_manager.get_packages("full"):
            if len(app_set.package_list) > 1:
                nikgapps_config_lines += "\n# Set " + app_set.title + "=0 if you want to skip installing all " \
                                                                      "packages belonging to " \
                                                                      "" + app_set.title + " Package\n"
                nikgapps_config_lines += app_set.title + "=" + "1" + "\n"
                for pkg in app_set.package_list:
                    nikgapps_config_lines += ">>" + pkg.package_title + "=" + str(pkg.enabled) + "\n"
                nikgapps_config_lines += "\n"
            else:
                for pkg in app_set.package_list:
                    nikgapps_config_lines += pkg.package_title + "=" + str(pkg.enabled) + "\n"
        for app_set in self.package_manager.get_packages("go"):
            if len(app_set.package_list) > 1:
                nikgapps_config_lines += "\n# Set " + app_set.title + "=0 if you want to skip installing all " \
                                                                      "packages belonging to " \
                                                                      "" + app_set.title + " Package\n"
                nikgapps_config_lines += app_set.title + "=" + "1" + "\n\n"
                nikgapps_config_lines += "# Setting CoreGo=0 will not skip following packages," \
                                         " set them to 0 if you want to skip them  \n"
            else:
                nikgapps_config_lines += app_set.title + "=" + "1" + "\n"
        nikgapps_config_lines += "\n"
        nikgapps_config_lines += "# Following are the Addon packages NikGapps supports\n"
        for app_set in self.package_manager.get_packages("addons"):
            nikgapps_config_lines += app_set.title + "=" + "1" + "\n"
        return nikgapps_config_lines

    def get_config_packages(self):
        config_dict = self.config_dict
        app_set_list = []
        pre_defined_addons = {addon.title for addon in self.package_manager.get_packages("addons")}
        for app_set in self.package_manager.get_packages("all"):
            app_set: AppSet
            app_set_title = app_set.title
            app_set_config_value = self.config_dict.get(app_set_title)
            if not app_set_config_value:
                continue
            pkg_len = len(app_set.package_list)
            new_app_set = None
            if pkg_len > 1 and str(app_set_config_value) == "1":
                new_app_set = AppSet(app_set.title)
                for pkg in app_set.package_list:
                    pkg: Package
                    if app_set.title.lower() == "corego":
                        new_app_set.add_package(pkg)
                        if pkg.package_title != "ExtraFilesGo":
                            self.config_dict[str(">>" + pkg.package_title)] = "1"
                        continue
                    if str(">>" + pkg.package_title) not in config_dict:
                        if app_set.title in pre_defined_addons:
                            new_app_set.add_package(pkg)
                        continue
                    if config_dict[str(">>" + pkg.package_title)] in ("1", "2"):
                        new_app_set.add_package(pkg)
            elif pkg_len == 1 and str(config_dict[app_set.title]) in ("1", "2"):
                new_app_set = app_set
            if new_app_set is not None:
                app_set_list.append(new_app_set)
        return app_set_list

    def get_nikgapps_config(self, config_dict=None, config_objects=None, for_release=False):
        nikgapps_config_lines = "# NikGapps configuration file\n\n"
        nikgapps_config_lines += "# If you are not sure about the config, " \
                                 "just skip making changes to it or comment it by adding # before it\n"
        nikgapps_config_lines += "# visit https://nikgapps.com/misc/2022/02/22/" \
                                 "NikGapps-Config.html to read everything about nikgapps\n\n"

        for config_obj in self.config_objects if config_objects is None else config_objects:
            nikgapps_config_lines += config_obj.get_string()

        nikgapps_config_lines += "# Following are the packages with default configuration\n"
        config_dict = self.config_dict if config_dict is None else config_dict
        for app_set in self.package_manager.get_packages("full"):
            if len(app_set.package_list) > 1:
                nikgapps_config_lines += "\n# Set " + app_set.title + "=0 if you want to skip installing all " \
                                                                      "packages belonging to " \
                                                                      "" + app_set.title + " Package\n"
                nikgapps_config_lines += app_set.title + "=" + config_dict.get(app_set.title, "0") + "\n"
                for pkg in app_set.package_list:
                    entry = ">>" + pkg.package_title
                    nikgapps_config_lines += entry + "=" + config_dict.get(entry, "0") + "\n"
                nikgapps_config_lines += "\n"
            else:
                for pkg in app_set.package_list:
                    entry = pkg.package_title
                    nikgapps_config_lines += entry + "=" + config_dict.get(entry, "0") + "\n"
        for app_set in self.package_manager.get_packages("go"):
            if len(app_set.package_list) > 1:
                nikgapps_config_lines += "\n# Set " + app_set.title + "=0 if you want to skip installing all " \
                                                                      "packages belonging to " \
                                                                      "" + app_set.title + " Package\n"
                nikgapps_config_lines += app_set.title + "=" + config_dict.get(app_set.title, "0") + "\n\n"
                nikgapps_config_lines += "# Setting CoreGo=0 will not skip following packages," \
                                         " set them to 0 if you want to skip them  \n"
            else:
                nikgapps_config_lines += app_set.title + "=" + config_dict.get(app_set.title, "0") + "\n"
        nikgapps_config_lines += "\n"
        nikgapps_config_lines += "# Following are the Addon packages NikGapps supports\n"
        for app_set in self.package_manager.get_packages("addons"):
            nikgapps_config_lines += app_set.title + "=" + config_dict.get(app_set.title, "0") + "\n"
        if for_release:
            nikgapps_config_lines += "\n"
            nikgapps_config_lines += "# NikGapps debloater starts here, add all the stuff to add to debloater.config below (for elite and user builds only), check examples below\n"
            nikgapps_config_lines += "# YouTube\n"
            nikgapps_config_lines += "# /system/app/YouTube\n"
            nikgapps_config_lines += "\n"
        return nikgapps_config_lines

    def get_debloater_list(self):
        lines = []
        start_reading = False
        for line in self.raw_config.splitlines():
            if not start_reading and not line.__contains__("NikGapps debloater starts here"):
                continue
            start_reading = True
            if line.__eq__('') or line.__eq__('\n') or line.startswith('#') or line.__contains__("="):
                continue
            lines.append(line.replace('\n', '').strip())
        return lines

    def describe_nikgapps_config(self, get_list=False):
        list_of_apps = []
        if len(self.config_package_list) == 0:
            return "No packages are enabled to be installed." if not get_list else list_of_apps
        result = "Following Apps will be installed:\n"
        for appset in self.config_package_list:
            appset: AppSet
            result += f"\n-> {appset.title}  "
            list_of_apps.append(f"-> {appset.title}")
            if len(appset.package_list) >= 1:
                result += "\n"
                for package in appset.package_list:
                    result += f"- {package.package_title}  \n"
                    list_of_apps.append(f"- {package.package_title}")
        return result.strip() if not get_list else list_of_apps

    def upload_nikgapps_config(self):
        analytics_dict = {}
        key = "config_version_" + Statics.get_android_code(self.android_version)
        analytics_dict[key] = str(self.config_version)
        tracker_repo = GitOp.setup_tracker_repo()
        repo_dir = tracker_repo.working_tree_dir
        if F.dir_exists(repo_dir):
            print(f"{repo_dir} exists!")
            config_version_json = repo_dir + Statics.dir_sep + "config_version.json"
            if F.file_exists(config_version_json):
                print("File Exists!")
                custom_builds_json_string = ""
                for line in F.read_string_file(config_version_json):
                    custom_builds_json_string += line
                print(custom_builds_json_string)
                print()
                decoded_hand = json.loads(custom_builds_json_string)
                if decoded_hand.get(key) is not None:
                    version_on_server = decoded_hand[key]
                    print("version on server is: " + version_on_server)
                    if int(version_on_server) < int(self.config_version):
                        print("server needs updating to " + self.config_version)
                        if self.upload():
                            decoded_hand[key] = str(self.config_version)
                            with open(config_version_json, "w") as file:
                                json.dump(decoded_hand, file, indent=2)
                        else:
                            print("Cannot update the tracker since config file failed to upload")
                    elif int(version_on_server) == int(self.config_version):
                        print("server is in sync")
                    else:
                        print("server is on higher version")
                else:
                    print(f"{key} key doesn't exist! creating the file with the key")
                    if self.upload():
                        decoded_hand[key] = str(self.config_version)
                        with open(config_version_json, 'w') as f:
                            json.dump(decoded_hand, f, indent=2)
                            print(f"{config_version_json} file is created")
                    else:
                        print("Cannot update the tracker since config file failed to upload")

            else:
                print(config_version_json + " doesn't exist! creating the file")
                if self.upload():
                    with open(config_version_json, 'w') as f:
                        json.dump(analytics_dict, f, indent=2)
                        print(f"{config_version_json} file is created")
                else:
                    print("Cannot update the tracker since config file failed to upload")
            if tracker_repo.due_changes():
                print("Updating the config version to v" + str(self.config_version))
                tracker_repo.git_push("Updating config version to v" + str(self.config_version),
                                      push_untracked_files=True)
            else:
                print("There is no change in config version to update!")
        else:
            print(f"{repo_dir} doesn't exist!")

    def upload(self):
        execution_status = False
        # create nikgapps.config file and upload to sourceforge
        temp_nikgapps_config_location = Statics.get_temp_packages_directory(
            self.android_version) + Statics.dir_sep + "nikgapps.config"
        F.write_string_in_lf_file(self.get_nikgapps_config(for_release=True), temp_nikgapps_config_location)
        if F.file_exists(temp_nikgapps_config_location):
            release_dir = "NikGappsConfigs"
            u = Upload(android_version=self.android_version, release_type=release_dir, upload_files=Config.UPLOAD_FILES)
            file_type = "nikgappsconfig"
            remote_directory = u.get_cd(file_type) + "/v" + str(self.config_version)
            execution_status, download_link, file_size_mb = u.upload(file_name=temp_nikgapps_config_location,
                                                                     remote_directory=remote_directory)
            u.close_connection()
        return execution_status

    def get_release_date(self):
        release_date = None
        if self.config_path is not None:
            file_contents = F.read_string_file(self.config_path)
            for line in file_contents:
                if line.startswith("RELEASE_DATE="):
                    release_date = line.split("=")[1].strip()
                    break
        return release_date

    def update_with_release_date(self, release_date=None):
        if release_date is None:
            release_date = Requests.get_release_date(self.android_version, Config.RELEASE_TYPE)
        if self.config_path is not None:
            file_contents = F.read_string_file(self.config_path)
            for index, line in enumerate(file_contents):
                if line.startswith("RELEASE_DATE="):
                    file_contents[index] = f"RELEASE_DATE={release_date}\n"
                    break
                if line.startswith("Version="):
                    file_contents.insert(index, f"RELEASE_DATE={release_date}\n")
                    break
            file_string = "".join(file_contents)
            F.write_string_in_lf_file(file_string, self.config_path)
        else:
            print("Invalid config path!")



from NikGapps.helper.overlay.Bool import Bool
from NikGapps.helper.overlay.Resources import Resources
from NikGapps.helper.overlay.String import String
from NikGapps.helper.overlay.StringArray import StringArray


class Library:

    @staticmethod
    def get_google_dialer_resources():
        r = Resources()
        r.add(String('config_defaultDialer', 'com.google.android.dialer',
                     comment='The name of the package that will hold the dialer role by default.'))
        r.add(String('dialer_default_class', 'com.google.android.dialer.extensions.GoogleDialtactsActivity',
                     comment='Class name for the default main Dialer activity'))
        r.add(String('incall_default_class', 'com.android.incallui.InCallServiceImpl'))
        r.add(String('config_defaultCallRedirection', 'com.google.android.dialer'))
        r.add(String('config_defaultCallScreening', 'com.google.android.dialer'))
        r.add(Bool('grant_location_permission_enabled', True,
                   comment='Determines if the granting of temporary location permission to the default dialer '
                           'during an emergency call should be allowed.'))
        r.add(Bool('call_recording_enabled', True))
        r.add(StringArray('config_globallyDisabledComponents', [
            'com.google.android.dialer/com.android.dialer.rtt.settings.impl.gateway.RttSettingsDeepLink']))
        return r

    @staticmethod
    def get_android_auto_resources():
        r = Resources()
        r.add(String("config_systemAutomotiveProjection", "com.google.android.projection.gearhead",
                     comment="The names of the packages that will hold the system automotive projection role."))
        return r

    @staticmethod
    def get_google_contacts_resources():
        r = Resources()
        r.add(String('config_systemContacts', 'com.google.android.contacts',
                     comment='The name of the package that will hold the contacts role by default.'))
        return r

    @staticmethod
    def get_digital_wellbeing_resources():
        r = Resources()
        r.add(String('config_systemWellbeing', 'com.google.android.apps.wellbeing',
                     comment='The name of the package that will hold the wellbeing role by default.'))
        r.add(String('config_defaultWellbeingPackage', 'com.google.android.apps.wellbeing',
                     comment='The package name for the default wellbeing application.'))
        r.add(String('config_defaultDndAccessPackages', 'com.google.android.gms:com.google.android.apps.wellbeing',
                     comment='Colon separated list of package names that should be granted DND access.'))
        r.add(StringArray('config_packagesExemptFromSuspension', ['com.google.android.apps.wellbeing']))
        r.add(StringArray('config_quickSettingsAutoAdd', ['wind_down_first_time_setup:custom(com.google.android.apps.wellbeing/.screen.ui.GrayscaleTileService)',
                                                          'focus_mode_first_time_setup:custom(com.google.android.apps.wellbeing/.focusmode.quicksettings.FocusModeTileService)']))
        return r

    @staticmethod
    def get_flipendo_resources():
        r = Resources()
        r.add(String('config_powerSaveModeChangedListenerPackage', 'com.google.android.flipendo'))
        return r

    @staticmethod
    def get_gms_core_resources():
        r = Resources()
        r.add(String('config_defaultDndAccessPackages', 'com.google.android.gms:com.google.android.apps.wellbeing',
                     comment='Colon separated list of package names that should be granted DND access.'))
        r.add(String('config_defaultSupervisionProfileOwnerComponent',
                     'com.google.android.gms/.kids.account.receiver.ProfileOwnerReceiver'))
        r.add(String('config_systemSupervision', 'com.google.android.gms.supervision'))
        r.add(String('def_backup_transport', 'com.google.android.gms/.backup.BackupTransportService'))
        r.add(String('metadata_sync_pacakge', 'com.google.android.gms'))
        r.add(String('config_defaultAutofillService', 'com.google.android.gms/.autofill.service.AutofillService'))
        r.add(String('config_defaultNearbySharingComponent',
                     'com.google.android.gms/com.google.android.gms.nearby.sharing.ShareSheetActivity'))
        r.add(String('config_defaultNearbyFastPairSettingsDevicesComponent',
                     'com.google.android.gms/com.google.android.gms.nearby.discovery.devices.SavedDevicesSettingsActivity'))
        r.add(String('config_defaultNetworkRecommendationProviderPackage', 'com.google.android.gms'))
        r.add(String('config_persistentDataPackageName', 'com.google.android.gms'))
        r.add(String('config_systemCompanionDeviceProvider', 'com.google.android.gms'))
        r.add(String('config_appsAuthorizedForSharedAccounts', ';com.android.vending;com.android.settings;'))
        r.add(String('config_deviceConfiguratorPackageName', 'com.google.android.gms'))
        r.add(String('config_systemGameService', 'com.google.android.gms'))
        r.add(String('config_systemActivityRecognizer', 'com.google.android.gms'))
        r.add(String('config_devicePolicyManagementUpdater', 'com.google.android.gms'))
        r.add(String('platform_number_verification_package', 'com.google.android.gms'))
        r.add(String('config_defaultQrCodeComponent',
                     'com.google.android.gms/.mlkit.barcode.ui.PlatformBarcodeScanningActivityProxy'))
        r.add(String('config_defaultCredentialManagerHybridService', 'com.google.android.gms/.auth.api.credentials.credman.service.RemoteService'))
        r.add(String('config_remoteCopyPackage', 'com.google.android.gms/.nearby.sharing.RemoteCopyShareSheetActivity'))
        r.add(String('config_systemCallStreaming', 'com.google.android.gms'))
        r.add(String('config_oemCredentialManagerDialogComponent', 'com.google.android.gms/.identitycredentials.ui.CredentialChooserActivity'))
        r.add(Bool('config_enableAutoPowerModes', True))
        r.add(Bool('config_enableNetworkLocationOverlay', True))
        r.add(Bool('config_enableFusedLocationOverlay', True))
        r.add(Bool('config_enableGeocoderOverlay', True))
        r.add(Bool('config_enableGeofenceOverlay', True))
        r.add(Bool('vvm3_enabled', True))
        r.add(StringArray('config_priorityOnlyDndExemptPackages',
                          ['com.android.dialer', 'com.google.android.dialer', 'com.android.server.telecom', 'android',
                           'com.android.systemui']))
        r.add(StringArray('config_accessibility_allowed_install_source', ['com.android.vending']))
        r.add(StringArray('config_forceQueryablePackages',
                          ['com.android.settings', 'com.android.providers.settings', 'com.android.vending',
                           'com.google.android.gms']))
        r.add(StringArray('config_integrityRuleProviderPackages', ['com.android.vending', 'com.google.android.gms']))
        r.add(StringArray('config_disabledUntilUsedPreinstalledImes', ['com.google.android.inputmethod.latin']))
        r.add(StringArray('config_locationProviderPackageNames',
                          ['com.google.android.gms', 'com.android.location.fused']))
        r.add(StringArray('config_ephemeralResolverPackage', ['com.google.android.gms']))
        r.add(StringArray('config_globallyDisabledComponents', [
            'com.android.vending/com.google.android.finsky.systemupdate.SystemUpdateSettingsContentProvider',
            'com.android.vending/com.google.android.finsky.systemupdateactivity.SettingsSecurityEntryPoint',
            'com.android.vending/com.google.android.finsky.systemupdateactivity.SystemUpdateActivity',
            'com.google.android.gms/com.google.android.gms.update.phone.PopupDialog',
            'com.google.android.gms/com.google.android.gms.update.OtaSuggestionSummaryProvider',
            'com.google.android.gms/com.google.android.gms.update.SystemUpdateActivity',
            'com.google.android.gms/com.google.android.gms.update.SystemUpdateGcmTaskService',
            'com.google.android.gms/com.google.android.gms.update.SystemUpdateService',
            'com.google.android.gms/com.google.android.gms.update.SystemUpdateService.ActiveReceiver',
            'com.google.android.gms/com.google.android.gms.update.SystemUpdateService.Receiver',
            'com.google.android.gms/com.google.android.gms.update.SystemUpdateService.SecretCodeReceiver',
            'com.google.android.gms/com.google.android.gms.chimera.GmsIntentOperationService$GmsExternalReceiver']))
        return r

    @staticmethod
    def get_google_location_history_resources():
        r = Resources()
        r.add(StringArray('config_locationProviderPackageNames',
                          ['com.google.android.gms', 'com.android.location.fused',
                           'com.google.android.gms.location.history']))
        return r

    @staticmethod
    def get_google_messages_resources():
        r = Resources()
        r.add(String('config_defaultSms', 'com.google.android.apps.messaging'))
        return r

    @staticmethod
    def get_google_photos_resources():
        r = Resources()
        r.add(String('config_systemGallery', 'com.google.android.apps.photos'))
        r.add(String('config_defaultGallery', 'com.google.android.apps.photos'))
        return r

    @staticmethod
    def get_google_talkback_resources():
        r = Resources()
        r.add(String('config_defaultAccessibilityService', 'com.google.android.marvin.talkback/.TalkBackService'))
        r.add(StringArray('config_preinstalled_screen_reader_services',
                          ['com.google.android.marvin.talkback/com.google.android.marvin.talkback.TalkBackService',
                           'com.google.android.marvin.talkback/com.google.android.accessibility.selecttospeak.SelectToSpeakService']))
        r.add(StringArray('config_preinstalled_interaction_control_services',
                          ['com.google.android.marvin.talkback/com.android.switchaccess.SwitchAccessService',
                           'com.google.android.marvin.talkback/com.google.android.accessibility.accessibilitymenu.AccessibilityMenuService']))
        return r

    @staticmethod
    def get_google_tts_resources():
        r = Resources()
        r.add(String('config_systemSpeechRecognizer', 'com.google.android.tts'))
        return r

    @staticmethod
    def get_pixel_launcher_resources():
        r = Resources()
        r.add(String('config_recentsComponentName',
                     'com.google.android.apps.nexuslauncher/com.android.quickstep.RecentsActivity'))
        r.add(String('config_secondaryHomePackage', 'com.google.android.apps.nexuslauncher'))
        r.add(String('exit_to_action_in_initial_setup',
                     'com.google.android.apps.nexuslauncher/com.android.quickstep.action.GESTURE_ONBOARDING_ALL_SET'))
        r.add(Bool('config_swipe_up_gesture_setting_available', True))
        r.add(Bool('config_custom_swipe_up_gesture_setting_available', True))
        r.add(StringArray('configs_base',
                          ['launcher/enable_one_search=true', 'launcher/ENABLE_LOCAL_RECOMMENDED_WIDGETS_FILTER=true',
                           'launcher/ENABLE_SMARTSPACE_ENHANCED=true',
                           'launcher/ENABLE_WIDGETS_PICKER_AIAI_SEARCH=true']))
        return r

    @staticmethod
    def get_velvet_resources():
        r = Resources()
        r.add(String('config_defaultAssistant', 'com.google.android.googlequicksearchbox'))
        r.add(String('config_defaultContextualSearchPackageName', 'com.google.android.googlequicksearchbox'))
        r.add(Bool('config_allowDisablingAssistDisclosure', True))
        return r

    @staticmethod
    def get_youtube_music_resources():
        r = Resources()
        r.add(String('config_defaultMusic', 'com.google.android.apps.youtube.music'))
        return r

    @staticmethod
    def get_template():
        return Resources()

    @staticmethod
    def get_device_health_services_resources():
        r = Resources()
        r.add(String('battery_suggestion_summary', 'Turn on Adaptive Battery'))
        r.add(String('smart_battery_manager_title', 'Adaptive Battery'))
        r.add(String('smart_battery_footer', 'When Battery Manager detects that apps are draining battery, you\u2019ll have the option to restrict these apps. Restricted apps may not work properly and notifications may be delayed.'))
        r.add(String('smart_battery_title', 'Use Adaptive Battery'))
        r.add(String('battery_tip_smart_battery_summary', 'Turn on Adaptive Battery'))
        return r

    @staticmethod
    def get_devices_personalization_services_resources():
        r = Resources()
        r.add(String('config_defaultAmbientContextConsentComponent',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.ambientcontext.ui.ConsentActivity'))
        r.add(String('config_defaultWallpaperEffectsGenerationService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.wallpapereffects.AiAiWallpaperEffectsGenerationService'))
        r.add(String('config_defaultAmbientContextDetectionService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiAmbientContextDetectionService'))
        r.add(String('config_defaultAppPredictionService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiPredictionService'))
        r.add(String('config_defaultAttentionService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.attention.service.AiAiAttentionService'))
        r.add(String('config_defaultSystemCaptionsManagerService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.captions.SystemCaptionsManagerService'))
        r.add(String('config_defaultAugmentedAutofillService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiAugmentedAutofillService'))
        r.add(String('config_defaultContentCaptureService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiContentCaptureService'))
        r.add(String('config_defaultListenerAccessPackages',
                     'com.android.launcher3:com.google.android.apps.nexuslauncher:com.google.android.apps.restore:com.google.android.setupwizard:com.google.android.apps.pixelmigrate:com.google.android.as:com.google.android.projection.gearhead'))
        r.add(String('config_defaultContentSuggestionsService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiContentSuggestionsService'))
        r.add(String('config_defaultRotationResolverService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiRotationResolverService'))
        r.add(String('config_defaultSystemCaptionsService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.captions.CaptionsService'))
        r.add(String('config_systemAmbientAudioIntelligence', 'com.google.android.as'))
        r.add(String('config_systemAudioIntelligence', 'com.google.android.as'))
        r.add(String('config_systemNotificationIntelligence', 'com.google.android.as'))
        r.add(String('config_defaultSearchUiService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiSearchUiService'))
        r.add(String('config_systemTextIntelligence', 'com.google.android.as'))
        r.add(String('config_systemUiIntelligence', 'com.google.android.as'))
        r.add(String('config_systemVisualIntelligence', 'com.google.android.as'))
        r.add(String('config_defaultSmartspaceService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiSmartspaceService'))
        r.add(String('config_defaultTranslationService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiTranslationService'))
        r.add(String('config_defaultAssistantAccessComponent',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.common.notification.service.AiAiNotificationAssistantService'))
        r.add(String('config_defaultOnDeviceSpeechRecognitionService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.AiAiSpeechRecognitionService'))
        r.add(Bool('config_adaptive_sleep_available', True))
        r.add(Bool('config_systemCaptionsServiceCallsEnabled', True))
        r.add(StringArray('configs_base', ['device_personalization_services/Captions__allow_box_x_axis_movement=true',
                                           'device_personalization_services/Captions__allow_use_public_speech_recognition=true',
                                           'device_personalization_services/Captions__allow_voip_call_without_audio_recording=true',
                                           'device_personalization_services/Captions__available_for_download=en-US;en-GB;en-CA;en-IE;en-AU;en-SG;en-IN;fr-FR;fr-CA;fr-BE;fr-CH;it-IT;it-CH;de-DE;de-AT;de-BE;de-CH;ja-JP;es-ES;es-US;cmn-Hant-TW;hi-IN;pt-BR;tr-TR;pl-PL;cmn-Hans-CN;ko-KR;ru-RU;vi-VN',
                                           'device_personalization_services/Captions__disable_prod=false',
                                           'device_personalization_services/Captions__enable_activation_boost=true',
                                           'device_personalization_services/Captions__enable_adaptive_box_width=true',
                                           'device_personalization_services/Captions__enable_augmented_modality=true',
                                           'device_personalization_services/Captions__enable_augmented_modality_input=true',
                                           'device_personalization_services/Captions__enable_augmented_modality_language_detection=true',
                                           'device_personalization_services/Captions__enable_augmented_music=true',
                                           'device_personalization_services/Captions__enable_drag_and_double_tap_to_resize=true',
                                           'device_personalization_services/Captions__enable_ec_integration=false',
                                           'device_personalization_services/Captions__enable_language_detection=true',
                                           'device_personalization_services/Captions__enable_new_model_version_advanced_2023=true',
                                           'device_personalization_services/Captions__enable_one_caption_experiment=false',
                                           'device_personalization_services/Captions__enable_punctuations=true',
                                           'device_personalization_services/Captions__enable_setting_page=false',
                                           'device_personalization_services/Captions__enable_survey=false',
                                           'device_personalization_services/Captions__enable_text_transform=true',
                                           'device_personalization_services/Captions__enable_westworld_logging=false',
                                           'device_personalization_services/Captions__enable_zero_day=false',
                                           'device_personalization_services/Captions__manifest_url_template=https://storage.googleapis.com/captions/%{NAMESPACE}_%{VERSION}_manifest.json',
                                           'device_personalization_services/Captions__model_version_v1=20190613',
                                           'device_personalization_services/Captions__model_version_v1_2=20200112',
                                           'device_personalization_services/Captions__new_model_version=20210623',
                                           'device_personalization_services/Captions__new_model_version_advanced_2023=20230808',
                                           'device_personalization_services/Captions__speech_threshold=0.2',
                                           'device_personalization_services/Captions__support_lang_id_disabled_after_dismissals=true',
                                           'device_personalization_services/Captions__supported_languages=en-US;fr-FR;it-IT;de-DE;ja-JP;es-ES;cmn-Hant-TW;hi-IN;pt-BR;tr-TR;pl-PL;cmn-Hans-CN;ko-KR;ru-RU;vi-VN',
                                           'device_personalization_services/Captions__supported_languages_beta_quality=ja-JP;pt-BR;tr-TR;pl-PL;cmn-Hans-CN;ko-KR;ru-RU;vi-VN',
                                           'device_personalization_services/Captions__surface_sound_events=true',
                                           'device_personalization_services/Captions__text_transform_augmented_input=true',
                                           'device_personalization_services/Captions__visibility_playing_duration_millis=5000',
                                           'device_personalization_services/Captions__visibility_stopped_duration_millis=1500',
                                           'device_personalization_services/Echo__search_enable_all_fallback_results=true',
                                           'device_personalization_services/Echo__search_enable_allowlist=true',
                                           'device_personalization_services/Echo__search_enable_app_fetcher_v2=true',
                                           'device_personalization_services/Echo__search_enable_app_search_tips=true',
                                           'device_personalization_services/Echo__search_enable_app_token_indexing=false',
                                           'device_personalization_services/Echo__search_enable_app_usage_stats_ranking=true',
                                           'device_personalization_services/Echo__search_enable_application_header_type=true',
                                           'device_personalization_services/Echo__search_enable_apps=true',
                                           'device_personalization_services/Echo__search_enable_appsearch_tips_ranking_improvement=true',
                                           'device_personalization_services/Echo__search_enable_assistant_quick_phrases_settings=true',
                                           'device_personalization_services/Echo__search_enable_bc_smartspace_settings=true',
                                           'device_personalization_services/Echo__search_enable_bc_translate_settings=true',
                                           'device_personalization_services/Echo__search_enable_eventstore=true',
                                           'device_personalization_services/Echo__search_enable_everything_else_above_web=true',
                                           'device_personalization_services/Echo__search_enable_fetcher_optimization_using_result_types=true',
                                           'device_personalization_services/Echo__search_enable_filter_pending_jobs=true',
                                           'device_personalization_services/Echo__search_enable_mdp_play_results=true',
                                           'device_personalization_services/Echo__search_enable_play=true',
                                           'device_personalization_services/Echo__search_enable_play_alleyoop=true',
                                           'device_personalization_services/Echo__search_enable_scraping=true',
                                           'device_personalization_services/Echo__search_enable_search_in_app_icon=true',
                                           'device_personalization_services/Echo__search_enable_settings_corpus=true',
                                           'device_personalization_services/Echo__search_enable_shortcuts=true',
                                           'device_personalization_services/Echo__search_enable_static_shortcuts=true',
                                           'device_personalization_services/Echo__search_enable_superpacks_app_terms=true',
                                           'device_personalization_services/Echo__search_enable_superpacks_play_results=true',
                                           'device_personalization_services/Echo__search_enable_top_hit_row=true',
                                           'device_personalization_services/Echo__search_enable_widget_corpus=true',
                                           'device_personalization_services/Echo__search_play_enable_spell_correction=true',
                                           'device_personalization_services/Cell__enable_cell=true',
                                           'device_personalization_services/Cell__enable_smartspace_events=true',
                                           'device_personalization_services/EchoSmartspace__check_notification_visibility=true',
                                           'device_personalization_services/EchoSmartspace__doorbell_when_for_update_time=true',
                                           'device_personalization_services/EchoSmartspace__enable_add_contextual_feedback_button_on_long_press=false',
                                           'device_personalization_services/EchoSmartspace__enable_add_internal_feedback_button=true',
                                           'device_personalization_services/EchoSmartspace__enable_agsa_settings_read=true',
                                           'device_personalization_services/EchoSmartspace__enable_cross_feature_rank_dedup_twiddler=true',
                                           'device_personalization_services/EchoSmartspace__enable_dimensional_logging=true',
                                           'device_personalization_services/EchoSmartspace__enable_encode_subcard_into_smartspace_target_id=true',
                                           'device_personalization_services/EchoSmartspace__enable_flight_landing_smartspace_aiai=true',
                                           'device_personalization_services/EchoSmartspace__enable_hotel_smartspace_aiai=true',
                                           'device_personalization_services/EchoSmartspace__enable_media_recs_for_driving=true',
                                           'device_personalization_services/EchoSmartspace__enable_predictor_expiration=true',
                                           'device_personalization_services/EchoSmartspace__enable_ring_channels_regex=true',
                                           'device_personalization_services/EchoSmartspace__enable_ring_using_ui_template=true',
                                           'device_personalization_services/EchoSmartspace__enable_travel_features_type_merge=true',
                                           'device_personalization_services/EchoSmartspace__ring_lockscreen_delay_seconds=0',
                                           'device_personalization_services/EchoSmartspace__ring_on_aod_only=true',
                                           'device_personalization_services/EchoSmartspace__runtastic_check_pause_action=true',
                                           'device_personalization_services/EchoSmartspace__runtastic_is_ongoing_default_true=true',
                                           'device_personalization_services/EchoSmartspace__smartspace_enable_daily_forecast=true',
                                           'device_personalization_services/EchoSmartspace__smartspace_enable_timely_reminder=true',
                                           'device_personalization_services/EchoSmartspace__strava_check_stop_action=true',
                                           'device_personalization_services/Echo__smartspace_dedupe_fast_pair_notification=true',
                                           'device_personalization_services/Echo__smartspace_doorbell_aiai_loading_screen=false',
                                           'device_personalization_services/Echo__smartspace_doorbell_allowlist_packages=com.nest.android, com.google.android.apps.chromecast.app',
                                           'device_personalization_services/Echo__smartspace_doorbell_loading_screen_state=4',
                                           'device_personalization_services/Echo__smartspace_enable_async_icon=true',
                                           'device_personalization_services/Echo__smartspace_enable_battery_notification_parser=true',
                                           'device_personalization_services/Echo__smartspace_enable_bedtime_active_predictor=true',
                                           'device_personalization_services/Echo__smartspace_enable_bedtime_reminder_predictor=true',
                                           'device_personalization_services/Echo__smartspace_enable_bluetooth_metadata_parser=true',
                                           'device_personalization_services/Echo__smartspace_enable_cross_device_timer=true',
                                           'device_personalization_services/Echo__smartspace_enable_dark_launch_outlook_events=true',
                                           'device_personalization_services/Echo__smartspace_enable_doorbell=true',
                                           'device_personalization_services/Echo__smartspace_enable_doorbell_context_wrapper=true',
                                           'device_personalization_services/Echo__smartspace_enable_doorbell_extras=true',
                                           'device_personalization_services/Echo__smartspace_enable_dwb_bedtime_predictor=true',
                                           'device_personalization_services/Echo__smartspace_enable_earthquake_alert_predictor=true',
                                           'device_personalization_services/Echo__smartspace_enable_echo_settings=true',
                                           'device_personalization_services/Echo__smartspace_enable_echo_unified_settings=true',
                                           'device_personalization_services/Echo__smartspace_enable_eta_doordash=true',
                                           'device_personalization_services/Echo__smartspace_enable_eta_lyft=true',
                                           'device_personalization_services/Echo__smartspace_enable_food_delivery_eta=true',
                                           'device_personalization_services/Echo__smartspace_enable_grocery=true',
                                           'device_personalization_services/Echo__smartspace_enable_light_off_predictor=true',
                                           'device_personalization_services/Echo__smartspace_enable_light_predictor=false',
                                           'device_personalization_services/Echo__smartspace_enable_media_wake_lock_acquire=true',
                                           'device_personalization_services/Echo__smartspace_enable_nap=true',
                                           'device_personalization_services/Echo__smartspace_enable_nudge=true',
                                           'device_personalization_services/Echo__smartspace_enable_outlook_events=true',
                                           'device_personalization_services/Echo__smartspace_enable_package_delivery=true',
                                           'device_personalization_services/Echo__smartspace_enable_paired_device_connections=true',
                                           'device_personalization_services/Echo__smartspace_enable_paired_device_predictor=true',
                                           'device_personalization_services/Echo__smartspace_enable_ridesharing_eta=true',
                                           'device_personalization_services/Echo__smartspace_enable_safety_check_predictor=true',
                                           'device_personalization_services/Echo__smartspace_enable_score_ranker=true',
                                           'device_personalization_services/Echo__smartspace_enable_sensitive_notification_twiddler=true',
                                           'device_personalization_services/Echo__smartspace_enable_step_predictor=true',
                                           'device_personalization_services/Echo__smartspace_enable_subcard_logging=true',
                                           'device_personalization_services/Echo__smartspace_gaia_twiddler=true',
                                           'device_personalization_services/Echo__smartspace_outlook_event_source_of_truth=TEXT_ONLY',
                                           'device_personalization_services/Echo__smartspace_package_delivery_card_delay_seconds=0',
                                           'device_personalization_services/Echo__smartspace_show_cross_device_timer_label=true',
                                           'device_personalization_services/Echo__smartspace_use_flashlight_action_chip=true',
                                           'device_personalization_services/Echo_smartspace__enable_flight_landing_smartspace_aiai=true',
                                           'device_personalization_services/Echo_smartspace__enable_hotel_smartspace_aiai=true',
                                           'device_personalization_services/Echo_smartspace__smartspace_enable_daily_forecast=true',
                                           'device_personalization_services/Echo_smartspace__smartspace_enable_timely_reminder=true',
                                           'device_personalization_services/Overview__enable_barcode_detection=false',
                                           'device_personalization_services/Overview__enable_image_search=false',
                                           'device_personalization_services/Overview__enable_image_selection=true',
                                           'device_personalization_services/Overview__enable_japanese_ocr=true',
                                           'device_personalization_services/Overview__enable_lens_r_overview_long_press=true',
                                           'device_personalization_services/Overview__enable_lens_r_overview_select_mode=true',
                                           'device_personalization_services/Overview__enable_lens_r_overview_translate_action=false',
                                           'device_personalization_services/Overview__enable_overview=true',
                                           'device_personalization_services/Overview__enable_pir_clearcut_logging=true',
                                           'device_personalization_services/Overview__enable_pir_westworld_logging=true',
                                           'device_personalization_services/Overview__enable_proactive_hints=false',
                                           'device_personalization_services/Overview__enable_superpacks_pir_protocol=true',
                                           'device_personalization_services/Overview__min_lens_agsa_app_version=301084649',
                                           'device_personalization_services/Translate__app_blocklist=com.google.android.talk',
                                           'device_personalization_services/Translate__blue_chip_translate_enabled=true',
                                           'device_personalization_services/Translate__characterset_lang_detection_enabled=true',
                                           'device_personalization_services/Translate__chat_translate_languages=de,en,es,fr,it,ja,hi,zh,ru,pl,pt,ko,th,tr,nl,zh_Hant,sv,da,vi,ar,fa,no,bn,fil',
                                           'device_personalization_services/Translate__copy_to_translate_enabled=true',
                                           'device_personalization_services/Translate__differentiate_simplified_and_traditional_chinese=true',
                                           'device_personalization_services/Translate__disable_translate_without_system_animation=false',
                                           'device_personalization_services/Translate__enable_chronicle_migration=true',
                                           'device_personalization_services/Translate__enable_default_langid_model=true',
                                           'device_personalization_services/Translate__enable_dictionary_langid_detection=true',
                                           'device_personalization_services/Translate__enable_opmv4_service=true',
                                           'device_personalization_services/Translate__enable_settings_backup_restore=false',
                                           'device_personalization_services/Translate__enable_setup_wizard_dialog_v2=false',
                                           'device_personalization_services/Translate__enable_spa_setting=false',
                                           'device_personalization_services/Translate__enable_translate_kit_api_migration=false',
                                           'device_personalization_services/Translate__interpreter_source_languages=de,en,ja,es,fr,it',
                                           'device_personalization_services/Translate__interpreter_target_languages=de,en,ja,es,fr,it',
                                           'device_personalization_services/Translate__replace_auto_translate_copied_text_enabled=true',
                                           'device_personalization_services/Translate__text_to_text_language_list=vi,ja,fa,ro,nl,mr,mt,ar,ms,it,eo,is,et,es,iw,zh,uk,af,id,ur,mk,cy,hi,el,be,pt,lt,hr,lv,hu,ht,te,de,bg,th,bn,tl,pl,tr,kn,sv,gl,ko,sw,cs,da,ta,gu,ka,sl,ca,sk,ga,sq,no,fi,ru,fr,en,zh_Hant,fil',
                                           'device_personalization_services/Translate__translation_service_enabled=true',
                                           'device_personalization_services/Translate__translator_expiration_enabled=true',
                                           'device_personalization_services/Translate__use_translate_kit_streaming_api=true',
                                           'device_personalization_services/SmartDictation__enable_alternatives_from_past_corrections=true',
                                           'device_personalization_services/SmartDictation__enable_alternatives_from_speech_hypotheses=true',
                                           'device_personalization_services/SmartDictation__enable_biasing_for_commands=true',
                                           'device_personalization_services/SmartDictation__enable_biasing_for_contact_fields=true',
                                           'device_personalization_services/SmartDictation__enable_biasing_for_contacts=true',
                                           'device_personalization_services/SmartDictation__enable_biasing_for_contacts_learned_from_past_corrections=true',
                                           'device_personalization_services/SmartDictation__enable_biasing_for_interests_model=true',
                                           'device_personalization_services/SmartDictation__enable_biasing_for_past_correction=true',
                                           'device_personalization_services/SmartDictation__enable_biasing_for_past_corrections=true',
                                           'device_personalization_services/SmartDictation__enable_biasing_for_screen_context=true',
                                           'device_personalization_services/SmartDictation__enable_personalized_biasing_on_locked_device=false',
                                           'device_personalization_services/SmartDictation__enable_selection_filtering=true',
                                           'device_personalization_services/SmartDictation__enable_stronger_boost_for_generic_phrases_biasing=false',
                                           'device_personalization_services/SmartRecCompose__enable_aiai_tc_generator=true',
                                           'device_personalization_services/SmartRecCompose__enable_compose_action_filter=true',
                                           'device_personalization_services/SmartRecCompose__enable_compose_tc=true',
                                           'device_personalization_services/SmartRecCompose__enable_deep_clu_model=true',
                                           'device_personalization_services/SmartRecOverviewChips__enable_action_boost_generator=true',
                                           'device_personalization_services/SmartRecOverviewChips__enable_assistant_geller_data_index=false',
                                           'device_personalization_services/SmartRecOverviewChips__enable_matchmaker_generator=true',
                                           'device_personalization_services/SmartRecOverviewChips__enable_reflection_generator=true',
                                           'device_personalization_services/SmartRecOverviewChips__enable_settings_card_generator=true',
                                           'device_personalization_services/SmartRecOverviewChips__enable_smartrec_for_overview_chips=true',
                                           'device_personalization_services/SmartRecOverviewChips__nasa_superpacks_manifest_url=https://www.gstatic.com/nasa/apps/pack_enus_10062022/22100618/2/en-us/superpacks_manifest.zip',
                                           'device_personalization_services/SmartRecOverviewChips__nasa_superpacks_manifest_ver=2022100601',
                                           'device_personalization_services/SmartRecPixelSearch__enable_aiai_tc_generator=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_all_fallbacks=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_appaction_generator=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_assistant_geller_data_index=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_assistant_generator=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_assistant_personalized_deeplinks=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_assistant_vertical_generator=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_chrometab_generator=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_corpora_via_search_context=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_entity_annotation_generator=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_entity_based_action_generation=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_feedback_ranking=false',
                                           'device_personalization_services/SmartRecPixelSearch__enable_gboard_suggestion=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_nasa_for_search=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_navigational_sites_generator=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_screenshot_generator=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_screenshot_thumbnail_cache=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_search_on_contacts=true',
                                           'device_personalization_services/SmartRecPixelSearch__enable_search_on_files=false',
                                           'device_personalization_services/SmartRecPixelSearch__enable_spelling_correction=true',
                                           'device_personalization_services/SmartRecPixelSearch__spelling_checker_frequency_score_overrides_map={"8":-7}',
                                           'device_personalization_services/NowPlaying__ambient_music_apk_music_detector_min_score=0.45',
                                           'device_personalization_services/NowPlaying__ambient_music_download_album_art_via_astrea=false',
                                           'device_personalization_services/NowPlaying__ambient_music_enable_resource_download_logging=false',
                                           'device_personalization_services/NowPlaying__ambient_music_enhanced_metadata_shards_manifest=4003:https://storage.googleapis.com/music-iq-db/updatable_db_v4/20241015-000033/manifest.json',
                                           'device_personalization_services/NowPlaying__ambient_music_get_model_state_cooldown_seconds_cloud_search=9',
                                           'device_personalization_services/NowPlaying__ambient_music_index_manifest_17_09_02=3105:https://storage.googleapis.com/music-iq-db/updatable_db_v3/20241013-030037/manifest.json',
                                           'device_personalization_services/NowPlaying__ambient_music_matcher_match_threshold_17_09_02=1.0E-6',
                                           'device_personalization_services/NowPlaying__ambient_music_on_demand_enabled=true',
                                           'device_personalization_services/NowPlaying__ambient_music_on_demand_music_confidence=0.48',
                                           'device_personalization_services/NowPlaying__ambient_music_on_demand_search_use_new_opt_in_flow=false',
                                           'device_personalization_services/NowPlaying__ambient_music_show_album_art=false',
                                           'device_personalization_services/NowPlaying__ambient_music_support_dsp_model_tflite_p6=true',
                                           'device_personalization_services/NowPlaying__ambient_music_use_enhanced_matches_database=false',
                                           'device_personalization_services/NowPlaying__ambient_music_use_metadata_shards_database=false',
                                           'device_personalization_services/NowPlaying__ambient_music_use_yt_domain_fife_urls=false',
                                           'device_personalization_services/NowPlaying__cloud_api_allowed=true',
                                           'device_personalization_services/NowPlaying__create_shortcut_on_np_enabled=false',
                                           'device_personalization_services/NowPlaying__default_music_player_setting=false',
                                           'device_personalization_services/NowPlaying__dsp_model_checksum_enabled=true',
                                           'device_personalization_services/NowPlaying__favorites_enabled=true',
                                           'device_personalization_services/NowPlaying__feature_users_count_enabled=true',
                                           'device_personalization_services/NowPlaying__handle_ambient_music_results_with_history=true',
                                           'device_personalization_services/NowPlaying__min_training_interval_millis=86400000',
                                           'device_personalization_services/NowPlaying__nnfp_v3_model_enabled=true',
                                           'device_personalization_services/NowPlaying__on_demand_enable_eager_prompt=true',
                                           'device_personalization_services/NowPlaying__on_demand_fingerprinter_being_setup_warning=true',
                                           'device_personalization_services/NowPlaying__on_demand_hide_if_fingerprinter_install_not_confirmed=true',
                                           'device_personalization_services/NowPlaying__on_demand_min_supported_aga_version=12.35.17',
                                           'device_personalization_services/NowPlaying__on_demand_retry_fingerprinter_install=true',
                                           'device_personalization_services/NowPlaying__shortcut_direct_create_enabled=false',
                                           'device_personalization_services/NowPlaying__youtube_export_enabled=true',
                                           'device_personalization_services/AdaptiveAudio__enable_adaptive_audio=true',
                                           'device_personalization_services/AdaptiveAudio__show_promo_notification=false',
                                           'device_personalization_services/AdaptiveAudio__use_silence_detector_state_bug_fix=true',
                                           'device_personalization_services/Attention__accel_sensor_enabled=false',
                                           'device_personalization_services/Attention__accel_sensor_threshold_mss=0.2',
                                           'device_personalization_services/Attention__enabled=true',
                                           'device_personalization_services/Attention__margin_horizontal_px=1000',
                                           'device_personalization_services/Attention__margin_vertical_px=1000',
                                           'device_personalization_services/Attention__proximity_sensor_enabled=false',
                                           'device_personalization_services/OverviewFederatedAnalytics__enable_fa=false',
                                           'device_personalization_services/OverviewFederatedAnalytics__enable_min_training_interval=false',
                                           'device_personalization_services/OverviewFederatedAnalytics__enable_non_synthetic_logs=false',
                                           'device_personalization_services/Autofill__enable=false',
                                           'device_personalization_services/Autofill__enable_fa=false',
                                           'device_personalization_services/Fedex__enable_fedex=false',
                                           'device_personalization_services/Logging__enable_aiai_clearcut_logging=false',
                                           'device_personalization_services/NotificationAssistant__enable_service=false',
                                           'device_personalization_services/VisualCortex__enable=false',
                                           'notification_assistant/generate_actions=true',
                                           'notification_assistant/generate_replies=true']))
        r.add(StringArray('config_globallyDisabledComponents', [
            'com.google.android.as/com.google.intelligence.sense.ambientmusic.history.HistoryContentProvider',
            'com.google.android.as/com.google.intelligence.sense.ambientmusic.history.HistoryActivity',
            'com.google.android.as/com.google.intelligence.sense.ambientmusic.AmbientMusicSettingsActivity',
            'com.google.android.as/com.google.intelligence.sense.ambientmusic.AmbientMusicNotificationsSettingsActivity',
            'com.google.android.as/com.google.intelligence.sense.ambientmusic.AmbientMusicSetupWizardActivity']))
        return r

    @staticmethod
    def get_settings_services_resources(android_version):
        r = Resources()
        r.add(String('config_systemSettingsIntelligence', 'com.google.android.settings.intelligence'))
        if float(android_version) >= 14:
            r.add(String('config_settingsintelligence_package_name', 'com.google.android.settings.intelligence'))
            r.add(
                String('config_settingsintelligence_log_action', 'com.google.android.settings.intelligence.LOG_BRIDGE'))
        return r

    @staticmethod
    def get_google_clock_resources():
        r = Resources()
        r.add(String('widget_default_package_name', 'com.google.android.deskclock'))
        r.add(String('widget_default_class_name', 'com.android.alarmclock.DigitalAppWidgetProvider'))
        r.add(String('config_dreamsDefaultComponent', 'com.google.android.deskclock/com.android.deskclock.Screensaver'))
        return r

    @staticmethod
    def get_cinematic_effect_resources():
        r = Resources()
        r.add(String('config_defaultWallpaperEffectsGenerationService',
                     'com.google.android.as/com.google.android.apps.miphone.aiai.app.wallpapereffects.AiAiWallpaperEffectsGenerationService'))
        return r

    @staticmethod
    def get_google_files_resources():
        r = Resources()
        r.add(String('config_systemFilePicker', 'com.google.android.apps.nbu.files'))
        r.add(String('config_sceenshotWorkProfileFilesApp', 'com.google.android.apps.nbu.files/com.google.android.apps.nbu.files.home.HomeActivity'))
        return r

    @staticmethod
    def get_google_wallpaper_resources():
        r = Resources()
        r.add(String('config_wallpaper_picker_package', 'com.google.android.apps.wallpaper'))
        r.add(String('config_wallpaperCropperPackage', 'com.google.android.apps.wallpaper'))
        r.add(String('config_wallpaper_picker_class', 'com.google.android.apps.wallpaper.picker.CategoryPickerActivity'))
        r.add(String('config_styles_and_wallpaper_picker_class', 'com.android.customization.picker.CustomizationPickerActivity'))
        return r

    @staticmethod
    def get_google_partner_setup_resources():
        r = Resources()
        r.add(String('config_deviceProvisioningPackage', 'com.google.android.apps.work.oobconfig'))
        return r

    @staticmethod
    def get_google_sounds_resources():
        r = Resources()
        r.add(String('config_sound_picker_package_name', 'com.google.android.soundpicker'))
        return r



import os

from NikGapps.build.PackageConstants import PackageConstants
from NikGapps.helper.AppSet import AppSet
from NikGapps.helper.Assets import Assets
from niklibrary.helper.Cmd import Cmd
from NikGapps.helper.Package import Package
from niklibrary.helper.Statics import Statics
from NikGapps.helper.overlay.Overlay import Overlay
from NikGapps.helper.overlay.Library import Library


class NikGappsManager:

    def __init__(self, android_version, arch="arm64"):
        self.android_version = android_version
        self.arch = arch
        self.source_directory = os.path.join(Statics.pwd,
                                             f"{android_version}_{arch}" if arch != "arm64" else f"{android_version}")

        self.cmd = Cmd()
        self.package_data = Assets.package_details
        self.appset_data = Assets.appsets_details
        self.extra_files_exceptions = {
            "extra.files": "ExtraFiles",
            "extra.files.go": "ExtraFilesGo",
            "ExtraFiles": "extra.files",
            "ExtraFilesGo": "extra.files.go"
        }
        self.extra_files_appsets = {
            "extra.files": "Core",
            "extra.files.go": "CoreGo"
        }

        self.package_to_appset_map = self._preprocess_appset_data()

    def _preprocess_appset_data(self):
        package_to_appset_map = {}
        for appset, versions in self.appset_data.items():
            for version_info in versions:
                packages = version_info["packages"]
                for package in packages:
                    if package not in package_to_appset_map:
                        package_to_appset_map[package] = []
                    package_to_appset_map[package].append(appset)
        return package_to_appset_map

    def get_packages(self, package_type):
        match package_type:
            case 'core':
                return self.get_core_package()
            case 'basic':
                return self.get_basic_package()
            case 'omni':
                return self.get_omni_package()
            case 'stock':
                return self.get_stock_package()
            case 'full':
                return self.get_full_package()
            case 'go':
                return self.get_go_package()
            case 'all':
                return self.get_all_packages()
            case _:
                return self.get_package_by_name_or_title(package_type)

    def get_package_by_name_or_title(self, package_type):
        if str(package_type).lower() == "addons":
            return self.get_addon_packages()
        if str(package_type).lower() == "addonsets":
            return self.get_addonsets()
        else:
            for app_set in self.get_full_package():
                if str(app_set.title).lower() == str(package_type).lower():
                    return [app_set]
                for package in app_set.package_list:
                    if str(package.package_title).lower() == str(package_type).lower():
                        return [AppSet(app_set.title, [package])]
            for app_set in self.get_go_package():
                if str(app_set.title).lower() == str(package_type).lower():
                    return [app_set]
                for package in app_set.package_list:
                    if str(package.package_title).lower() == str(package_type).lower():
                        return [AppSet(app_set.title, [package])]
            for app_set in self.get_addon_packages():
                if str(app_set.title).lower() == str(package_type).lower():
                    return [app_set]
                for package in app_set.package_list:
                    if str(package.package_title).lower() == str(package_type).lower():
                        return [AppSet(app_set.title, [package])]
        return [None]

    def get_package_details(self, package_name):
        if package_name in self.package_data:
            return self.package_data[package_name]
        else:
            raise ValueError(f"No details found for package: {package_name}")

    def find_appset_by_package(self, package_title, keyword=None, fallback_appset=None):
        matched_appsets = self.package_to_appset_map.get(package_title, [])

        if not matched_appsets:
            return None

        if keyword:
            for appset in matched_appsets:
                if keyword.lower() in appset.lower():
                    return appset

        if fallback_appset and fallback_appset in matched_appsets:
            return fallback_appset

        return matched_appsets[0]

    def create_appset_list(self, package_list, keyword=None, fallback_appset=None):
        appset_dict = {}
        for package in package_list:
            appset_title = self.find_appset_by_package(package.package_title, keyword=keyword,
                                                       fallback_appset=fallback_appset)
            if appset_title not in appset_dict:
                appset_dict[appset_title] = AppSet(appset_title, [package])
            else:
                appset_dict[appset_title].package_list.append(package)
        return list(appset_dict.values())

    def create_appset_list_from_packages(self, package_name_list, keyword=None, fallback_appset=None):
        appset_dict = {}
        for package_name in package_name_list:
            package = self.create_package(package_name)
            appset_title = self.find_appset_by_package(package_name, keyword=keyword, fallback_appset=fallback_appset)
            if appset_title not in appset_dict:
                appset_dict[appset_title] = AppSet(appset_title, [package])
            else:
                appset_dict[appset_title].package_list.append(package)
        return list(appset_dict.values())

    def create_package(self, package_name):
        if package_name in self.extra_files_exceptions:
            package = Package(
                title=package_name,
                package_name=self.extra_files_exceptions[package_name],
            )
        else:
            package_details = self.get_package_details(package_name)
            package_info = package_details[0]
            pkg_name = package_info['package_name']
            package = Package(
                title=package_info['title'],
                package_name=pkg_name,
                app_type=Statics.is_priv_app if package_info['type'] == "priv-app" else Statics.is_system_app,
                package_title=package_name,
                partition=package_info['partition'] if float(self.android_version) > 10 else "product"
            )
            if float(self.android_version) > 12:
                overlays = self.get_package_overlays(pkg_name)
                for overlay in overlays:
                    package.add_overlay(overlay)
            deletes = PackageConstants.get_package_deletes(pkg_name)
            for delete in deletes:
                package.delete(delete)
            package.clean_flash_only = PackageConstants.get_package_clean_flash_rule(pkg_name)
        script = PackageConstants.get_package_script(package_name)
        if script:
            package.additional_installer_script = script
        return package

    def get_package_overlays(self, package_name):
        if not float(self.android_version) > 12:
            return []
        package_overlays = {
            "com.google.android.gms": [
                {
                    "package_title": "GmsCore",
                    "package_name": "com.nikgapps.overlay.gmscore",
                    "resources": Library.get_gms_core_resources()
                }
            ],
            "com.google.android.dialer": [
                {
                    "package_title": "GoogleDialer",
                    "package_name": "com.nikgapps.overlay.dialer",
                    "resources": Library.get_google_dialer_resources()
                }
            ],
            "com.google.android.contacts": [
                {
                    "package_title": "GoogleContacts",
                    "package_name": "com.nikgapps.overlay.contacts",
                    "resources": Library.get_google_contacts_resources()
                }
            ],
            "com.google.android.tts": [
                {
                    "package_title": "GoogleTTS",
                    "package_name": "com.nikgapps.overlay.googletts",
                    "resources": Library.get_google_tts_resources()
                }
            ],
            "com.google.android.apps.wellbeing": [
                {
                    "package_title": "DigitalWellbeing",
                    "package_name": "com.nikgapps.overlay.wellbeing",
                    "resources": Library.get_digital_wellbeing_resources()
                }
            ],
            "com.google.android.marvin.talkback": [
                {
                    "package_title": "GoogleTalkback",
                    "package_name": "com.nikgapps.overlay.talkback",
                    "resources": Library.get_google_talkback_resources()
                }
            ],
            "com.google.android.flipendo": [
                {
                    "package_title": "Flipendo",
                    "package_name": "com.nikgapps.overlay.flipendo",
                    "resources": Library.get_flipendo_resources()
                }
            ],
            "com.google.android.apps.messaging": [
                {
                    "package_title": "GoogleMessages",
                    "package_name": "com.nikgapps.overlay.messages",
                    "resources": Library.get_google_messages_resources()
                }
            ],
            "com.google.android.gms.location.history": [
                {
                    "package_title": "GoogleLocationHistory",
                    "package_name": "com.nikgapps.overlay.googlelocationhistory",
                    "resources": Library.get_google_location_history_resources()
                }
            ],
            "com.google.android.apps.photos": [
                {
                    "package_title": "GooglePhotos",
                    "package_name": "com.nikgapps.overlay.googlephotos",
                    "resources": Library.get_google_photos_resources()
                }
            ],
            "com.google.android.settings.intelligence": [
                {
                    "package_title": "SettingsServices",
                    "package_name": "com.nikgapps.overlay.settingsintelligence",
                    "resources": Library.get_settings_services_resources(self.android_version)
                }
            ],
            "com.google.android.projection.gearhead": [
                {
                    "package_title": "AndroidAuto",
                    "package_name": "com.nikgapps.overlay.androidauto",
                    "resources": Library.get_android_auto_resources()
                }
            ],
            "com.google.android.apps.nexuslauncher": [
                {
                    "package_title": "PixelLauncher",
                    "package_name": "com.nikgapps.overlay.pixellauncher",
                    "resources": Library.get_pixel_launcher_resources()
                }
            ],
            "com.google.android.googlequicksearchbox": [
                {
                    "package_title": "Velvet",
                    "package_name": "com.nikgapps.overlay.googlequicksearchbox",
                    "resources": Library.get_velvet_resources()
                }
            ],
            "com.google.android.deskclock": [
                {
                    "package_title": "GoogleClock",
                    "package_name": "com.nikgapps.overlay.googleclock",
                    "resources": Library.get_google_clock_resources()
                }
            ],
            "com.google.android.as": [
                {
                    "package_title": "DevicePersonalizationServices",
                    "package_name": "com.nikgapps.overlay.ais",
                    "resources": Library.get_devices_personalization_services_resources()
                }
            ],
            "com.google.android.wallpaper.effects": [
                {
                    "package_title": "CinematicEffect",
                    "package_name": "com.nikgapps.overlay.cinematiceffect",
                    "resources": Library.get_cinematic_effect_resources()
                }
            ],
            "com.google.android.apps.youtube.music": [
                {
                    "package_title": "YouTubeMusic",
                    "package_name": "com.nikgapps.overlay.youtubemusic",
                    "resources": Library.get_youtube_music_resources()
                }
            ],
            "com.google.android.apps.turbo": [
                {
                    "package_title": "DeviceHealthServices",
                    "package_name": "com.nikgapps.overlay.turbo",
                    "resources": Library.get_device_health_services_resources()
                }
            ],
            "com.google.android.apps.nbu.files": [
                {
                    "package_title": "GoogleFiles",
                    "package_name": "com.nikgapps.overlay.files",
                    "resources": Library.get_google_files_resources()
                }
            ],
            "com.google.android.apps.wallpaper": [
                {
                    "package_title": "GoogleWallpaper",
                    "package_name": "com.nikgapps.overlay.wallpaper",
                    "resources": Library.get_google_wallpaper_resources()
                }
            ],
            "com.google.android.apps.work.oobconfig": [
                {
                    "package_title": "DeviceSetup",
                    "package_name": "com.nikgapps.overlay.partnersetup",
                    "resources": Library.get_google_partner_setup_resources()
                }
            ],
            "com.google.android.soundpicker": [
                {
                    "package_title": "GoogleSounds",
                    "package_name": "com.nikgapps.overlay.sounds",
                    "resources": Library.get_google_sounds_resources(),
                    "target_package": "com.android.settings"
                }
            ],
        }
        return [Overlay(apk_name=overlay["package_title"],
                        package_name=overlay["package_name"],
                        android_version=self.android_version,
                        resources=overlay["resources"],
                        target_package=overlay.get("target_package", "android"),
                        priority=overlay.get("priority", "99"))
                for overlay
                in package_overlays.get(package_name, [])]

    def get_go_package(self):
        go_packages = [
            "ExtraFilesGo",
            "GmsCore",
            "GooglePlayStore",
            "GoogleServicesFramework",
            "GoogleContactsSyncAdapter",
            "GoogleCalendarSyncAdapter",
            "GoogleGo",
            "AssistantGo",
            "MapsGo",
            "NavigationGo",
            "GalleryGo",
            "GmailGo"
        ]
        appset_list = self.create_appset_list_from_packages(go_packages, keyword="Go")
        return appset_list

    def get_core_package(self):
        core_packages = [
            "ExtraFiles",
            "GooglePlayStore",
            "GoogleServicesFramework",
            "GoogleContactsSyncAdapter",
            "GoogleCalendarSyncAdapter",
            "GmsCore"
        ]
        package_list = []
        for package_title in core_packages:
            package = self.create_package(package_title)
            package.addon_index = "05"
            package_list.append(package)
        return self.create_appset_list(package_list)

    def get_basic_package(self, delta=False):
        basic_packages = [
            "DigitalWellbeing",
            "GoogleMessages",
            "GoogleDialer",
            "GoogleContacts",
            "CarrierServices",
            "GoogleClock"
        ]
        appset_list = (self.get_core_package() if not delta else []) + self.create_appset_list_from_packages(
            basic_packages)
        return appset_list

    def get_omni_package(self, delta=False):
        setup_wizard_packages = [
            "SetupWizard",
            "GoogleRestore",
            "GoogleOneTimeInitializer"
        ]
        if float(self.android_version) < 12:
            setup_wizard_packages.append("AndroidMigratePrebuilt")
        setup_wizard = self.create_appset_list_from_packages(setup_wizard_packages, fallback_appset="SetupWizard")
        omni_packages = [
            "GoogleCalculator",
            "Drive",
            "GoogleMaps",
        ]
        if float(self.android_version) >= 11:
            omni_packages.append("GoogleLocationHistory")
        omni_packages.extend([
            "GooglePhotos",
            "DeviceHealthServices",
            "GBoard",
            "GoogleCalendar",
            "GoogleKeep"
        ])
        appset_list = ((self.get_basic_package() if not delta else [])
                       + setup_wizard
                       + self.create_appset_list_from_packages(omni_packages))
        return appset_list

    def get_stock_package(self, delta=False):
        pixel_launcher_packages = [
            "PixelLauncher",
            "DevicePersonalizationServices",
            "GoogleWallpaper"
        ]
        if float(self.android_version) >= 11:
            pixel_launcher_packages.append("QuickAccessWallet")
            if float(self.android_version) >= 12:
                pixel_launcher_packages.extend([
                    "SettingsServices",
                    "PrivateComputeServices"
                ])
                if float(self.android_version) >= 13:
                    pixel_launcher_packages.append("PixelThemes")
                    if float(self.android_version) >= 14:
                        # pixel_launcher_packages.append("CinematicEffect")
                        pixel_launcher_packages.append("EmojiWallpaper")
                        pixel_launcher_packages.append("PixelWeather")
                        if float(self.android_version) >= 15:
                            pixel_launcher_packages.append("AICore")
        pixel_specifics = self.create_appset_list_from_packages(pixel_launcher_packages)
        stock_packages = [
            "PlayGames",
            "GoogleRecorder",
            "GoogleFiles",
            "StorageManager"
        ]
        if float(self.android_version) >= 11:
            stock_packages.append("DocumentsUIGoogle")
        stock_packages.extend([
            "MarkupGoogle",
            "GoogleTTS",
            "Velvet",
            "Assistant",
            "GoogleSounds"
        ])
        return ((self.get_omni_package() if not delta else [])
                + pixel_specifics
                + self.create_appset_list_from_packages(stock_packages))

    def get_full_package(self, delta=False):
        full_packages = [
            "GoogleChrome",
            "WebViewGoogle",
            "TrichromeLibrary",
            "Gmail",
            "DeviceSetup",
            "AndroidAuto",
            "GoogleFeedback",
            "GooglePartnerSetup",
            "AndroidDevicePolicy"
        ]
        return (self.get_stock_package() if not delta else []) + self.create_appset_list_from_packages(full_packages)

    def get_addon_packages(self, addon_name=None):
        addon_packages = [
            "Meet",
            "GoogleDocs",
            "GoogleSheets",
            "GoogleSlides",
            "YouTube",
            "YouTubeMusic",
            "Books",
            "GoogleTalkback",
        ]
        if float(self.android_version) == 11:
            google_fi_packages = [
                "Tycho",
                "GCS"
            ]
            addon_packages.extend(google_fi_packages)
        # if float(self.android_version) == 13:
        #     addon_packages.append("PixelWallpapers")
        if float(self.android_version) >= 15:
            addon_packages.append("GooglePersonalSafety")
        pixel_setup_wizard_packages = [
            "SetupWizardPixel",
            "GoogleRestorePixel",
            "PixelSetupWizard",
            "GoogleOneTimeInitializerPixel"
        ]
        if float(self.android_version) < 12:
            pixel_setup_wizard_packages.append("AndroidMigratePixelPrebuilt")
        pixel_setup_wizard = self.create_appset_list_from_packages(pixel_setup_wizard_packages, keyword="Pixel")
        addon_set_list = pixel_setup_wizard + self.create_appset_list_from_packages(addon_packages)
        if addon_name:
            return [app_set for app_set in addon_set_list if app_set.title == addon_name]
        return addon_set_list

    def get_all_packages(self):
        all_package_list = []
        for app_set in self.get_full_package():
            all_package_list.append(app_set)
        for app_set in self.get_go_package():
            all_package_list.append(app_set)
        for app_set in self.get_addon_packages():
            all_package_list.append(app_set)
        return all_package_list

    def get_addonsets(self):
        addon_set_list = []
        for app_set in self.get_full_package():
            if app_set.title in ['Core', 'Pixelize']:
                continue
            addon_set_list.append(app_set)
        for app_set in self.get_go_package():
            if app_set.title in ['CoreGo']:
                continue
            addon_set_list.append(app_set)
        return addon_set_list



from .Assets import Assets
from niklibrary.helper.Statics import Statics
from niklibrary.helper.XmlOp import XmlOp
from .overlay.Overlay import Overlay


class Package:
    def __init__(self, title, package_name, app_type = None, package_title=None, partition=None):
        self.package_name = package_name
        self.title = title
        self.package_title = package_title
        if package_title is None:
            self.package_title = title
        self.partition = partition
        if partition is None:
            self.partition = "product"
        self.app_type = app_type  # Whether the Package is system app or private app
        # target_folder is the folder where the package will be installed
        if app_type == Statics.is_priv_app:
            self.target_folder = str(Statics.system_root_dir + "/" + "priv-app" + "/" + title).replace("\\\\", "/")
        elif app_type == Statics.is_system_app:
            self.target_folder = str(Statics.system_root_dir + "/" + "app" + "/" + title).replace("\\\\", "/")
        self.install_list = []  # Stores list of files installed to the package directory
        self.predefined_file_list = []  # Stores list of predefined file list
        self.overlay_list = []  # Stores list of overlay apks
        self.framework_list = []  # Stores list of Framework files
        self.primary_app_location = None  # This will help generate priv-app whitelist permissions
        self.folder_dict = dict()  # Stores list of folders that needs 755 permissions
        self.file_dict = dict()  # Stores the file location on server as key and on device as value
        self.delete_files_list = []  # Stores the path of file to delete. Helpful for removing AOSP counterpart
        self.priv_app_permissions = []  # Stores the priv-app whitelist permissions for the package
        self.delete_overlay_list = []  # Stores the list of overlays to delete
        self.enabled = 1
        self.validated = True
        self.clean_flash_only = False
        self.additional_installer_script = ""
        self.failure_logs = ""
        self.pkg_size = 0
        self.validation_script = None
        self.addon_index = "09"
        self.priv_app_permissions_str = None

    def add_overlay(self, overlay: Overlay):
        for overlay_item in self.overlay_list:
            if overlay_item.apk_name == overlay.apk_name:
                return
        self.overlay_list.append(overlay)

    def delete_overlay(self, overlay):
        if overlay not in self.delete_overlay_list:
            self.delete_overlay_list.append(overlay)

    def delete(self, data):
        if not str(data).startswith("/"):
            if data not in self.delete_files_list:
                self.delete_files_list.append(data)
        else:
            if data not in self.delete_files_list:
                self.delete_files_list.append(data)

    # this will generate the xml providing white-list permissions to the package
    def generate_priv_app_whitelist(self, app_set, permissions_list, android_version, pkg_path=None):
        for perm in self.priv_app_permissions:
            if perm not in permissions_list:
                permissions_list.append(perm)
        # permissions_path = "/etc/permissions/" + str(self.package_name) + ".xml"
        # import_path = Statics.get_import_path(app_set, self.package_title, permissions_path, android_version,
        #                                       pkg_path)
        # self.file_dict[import_path] = "etc/permissions/" + str(self.package_name) + ".xml"
        xml_op = XmlOp(self.package_name, permissions_list)
        return xml_op.to_string()

    def get_installer_script(self, pkg_size):
        lines = Assets.get_string_resource(Assets.installer_path)
        str_data = ""
        for line in lines:
            str_data += line

        str_data += "# Initialize the variables\n"
        str_data += "default_partition=\"" + self.partition + "\"\n"
        str_data += "clean_flash_only=\"" + str(self.clean_flash_only).lower() + "\"\n"
        str_data += "product_prefix=$(find_product_prefix \"$install_partition\")\n"
        str_data += "title=\"" + self.title + "\"\n"
        str_data += "package_title=\"" + self.package_title + "\"\n"
        str_data += "pkg_size=\"" + pkg_size + "\"\n"
        if self.package_name is not None:
            str_data += "package_name=\"" + self.package_name + "\"\n"
        else:
            str_data += "package_name=\"\"" + "\n"
        str_data += "packagePath=install" + self.package_title + "Files\n"
        str_data += "deleteFilesPath=delete" + self.package_title + "Files\n"
        str_data += "propFilePath=$(get_prop_file_path $package_title)\n"
        str_data += "\n"
        str_data += f"remove_aosp_apps_from_rom=\"\n"
        for delete_folder in self.delete_files_list:
            str_data += f"{delete_folder}\n"
        str_data += "\"\n"
        str_data += "\n"
        str_data += f"delete_overlays=\"\n"
        for delete_overlay in self.delete_overlay_list:
            str_data += f"{delete_overlay}\n"
        str_data += "\"\n"
        str_data += "\n"
        str_data += f"file_list=\"\n"
        for file in self.file_dict:
            str_data += self.file_dict[file] + "\n"
        if self.priv_app_permissions_str is not None:
            str_data += "___etc___permissions/" + self.package_name + ".xml\n"
        str_data += "\"\n"
        str_data += "\n"
        str_data += "remove_overlays() {\n"
        str_data += "   for i in $delete_overlays; do\n"
        str_data += "       delete_overlays \"$i\" \"$propFilePath\" \"$package_title\" \n"
        str_data += "   done\n"
        str_data += "}\n"
        str_data += "\n"
        str_data += "remove_existing_package() {\n"
        str_data += "   # remove the existing folder for clean install of " + self.package_title + "\n"
        str_data += "   delete_package \"" + self.title + "\" \"$package_title\" \n"
        # str_data += " # remove the data of package"
        # str_data += " delete_package_data \"" + self.package_name + "\"\n"
        str_data += "}\n"
        str_data += "\n"
        str_data += "remove_aosp_apps() {\n"
        str_data += "   # Delete the folders that we want to remove with installing " + self.package_title + "\n"
        str_data += "   for i in $remove_aosp_apps_from_rom; do\n"
        str_data += "       RemoveAospAppsFromRom \"$i\" \"$propFilePath\" \"$package_title\" \n"
        str_data += "   done\n"
        str_data += "}\n"
        str_data += "\n"
        str_data += "install_package() {\n"
        str_data += "   remove_existing_package\n"
        str_data += "   remove_aosp_apps\n"
        str_data += "   remove_overlays\n"
        str_data += "   # Create folders and set the permissions\n"
        for folder in self.folder_dict:
            str_data += "   make_dir \"" + folder + "\"\n"
        str_data += "\n"
        str_data += "   delete_prop_lines \"$propFilePath\"\n"
        str_data += "\n"
        str_data += "   # Copy the files and set the permissions\n"
        str_data += "   for i in $file_list; do\n"
        str_data += "       install_file \"$i\"\n"
        str_data += "   done\n"
        str_data += "\n"
        if not str(self.additional_installer_script).__eq__(""):
            str_data += self.additional_installer_script
            str_data += "\n"
        str_data += "   chmod 755 \"$COMMONDIR/addon.sh\";\n"
        str_data += "   update_prop \"$propFilePath\"" \
                    " \"install\"" \
                    " \"$propFilePath\" \"" + self.package_title + "\" \n"
        str_data += "   . $COMMONDIR/addon.sh \"" + self.package_title + "\" \"$propFilePath\" " + f"\"{self.addon_index}\"\n"
        str_data += "   copy_file \"$propFilePath\" \"$logDir/addonfiles/" + "$package_title.prop" + "\"\n"
        str_data += "}\n"
        str_data += "\n"
        str_data += self.validation_script + "\n" if self.validation_script is not None else "find_install_mode\n"
        str_data += "\n"
        return str_data

    def get_uninstaller_script(self):
        lines = Assets.get_string_resource(Assets.uninstaller_path)
        str_data = ""
        for line in lines:
            str_data += line
        str_data += "\n\n"
        str_data += "# Initialize the variables\n"
        str_data += "uninstall_addon=$1\n"
        str_data += "clean_flash_only=\"" + str(self.clean_flash_only).lower() + "\"\n"
        str_data += "title=\"" + self.title + "\"\n"
        str_data += "package_title=\"" + self.package_title + "\"\n"
        if self.package_name is not None:
            str_data += "package_name=\"" + self.package_name + "\"\n"
        else:
            str_data += "package_name=\"\"" + "\n"
        str_data += "\n"
        str_data += f"file_list=\"\n"
        for file in self.file_dict:
            str_data += self.file_dict[file] + "\n"
        if self.priv_app_permissions_str is not None:
            str_data += "___etc___permissions/" + self.package_name + ".xml\n"
        str_data += "\"\n"
        str_data += "\n"
        str_data += "uninstall_package"
        str_data += "\n"
        return str_data



from pathlib import Path
from importlib import resources

from niklibrary.helper.Assets import Assets as A
from niklibrary.helper.F import F
import os.path

from niklibrary.json.Json import Json


class Assets:
    with resources.files('NikGapps.helper').joinpath('assets') as asset_path:
        assets_folder = str(asset_path)
    if not F.dir_exists(assets_folder):
        assets_folder = os.path.join(os.getcwd(), 'assets')
    if not F.dir_exists(assets_folder):
        pwd = Path(os.getcwd()).parent
        for folders in pwd.iterdir():
            if folders.is_dir():
                if folders.name == "assets":
                    assets_folder = str(folders)
                    break
    cwd = assets_folder + os.path.sep
    system_name = A.system_name
    apksigner_path = A.get("apksigner.jar")
    apktool_path = A.get("apktool_2.10.0.jar")
    key_path = A.get("cert.pk8")
    cert_path = A.get("cert.x509.pem")
    private_key_pem = A.get("private_key.pem")
    aapt_path = A.aapt_path
    adb_path = A.adb_path
    addon_path = cwd + "addon.sh"
    header_path = cwd + "header.sh"
    functions_path = cwd + "functions.sh"
    busybox = A.get("busybox")
    file_sizes_path = cwd + "file_size.txt"
    mount_path = cwd + "mount.sh"
    mtg_mount_path = cwd + "mtg_mount.sh"
    unmount_path = cwd + "unmount.sh"
    nikgapps_functions = cwd + "nikgapps_functions.sh"
    update_script_path = cwd + "updater-script"
    nikgapps_config = cwd + "nikgapps.config"
    debloater_config = cwd + "debloater.config"
    installer_path = cwd + "installer.sh"
    uninstaller_path = cwd + "uninstaller.sh"
    changelog = cwd + "changelogs.yaml"
    sign_jar = os.path.join(A.assets_folder, "NikGappsZipSigner.jar")
    customize_path = cwd + "customize.sh"
    module_path = cwd + "module.prop"
    magisk_update_binary = cwd + "magisk-update-binary.sh"
    config_versions = Json.read_dict_from_file(cwd + "config_versions.json")
    package_details = Json.read_dict_from_file(cwd + "packages.json")
    appsets_details = Json.read_dict_from_file(cwd + "appsets.json")

    @staticmethod
    def get_string_resource(file_path):
        return F.read_string_file(file_path)

    @staticmethod
    def get_binary_resource(file_path):
        return F.read_binary_file(file_path)



#!/usr/bin/env python
from niklibrary.git.GitStatics import GitStatics

from NikGapps.build.Release import Release
from NikGapps.helper.Args import Args
from NikGapps.helper import Config
from niklibrary.helper.SystemStat import SystemStat
from niklibrary.helper.T import T
from niklibrary.compression.Modes import Modes
from niklibrary.git.GitOp import GitOp

from NikGapps.helper.git.TestGit import TestGit


def main():
    args = Args()
    print("Start of the Program")
    SystemStat.show_stats()
    t = T()
    android_versions = [Config.TARGET_ANDROID_VERSION]
    package_list = Config.BUILD_PACKAGE_LIST
    Config.UPLOAD_FILES = args.upload
    Config.USE_CACHED_APKS = args.use_cached_apks
    if Config.USE_CACHED_APKS:
        print("Using Cached Apks")
    Modes.DEFAULT = Modes.TAR_XZ if args.tar else Modes.ZIP
    if len(args.get_package_list()) > 0:
        package_list = args.get_package_list()

    if len(args.get_android_versions()) > 0:
        android_versions = args.get_android_versions()
    print("---------------------------------------")
    print("Android Versions to build: " + str(android_versions))
    print("---------------------------------------")
    print("Packages to build: " + str(package_list))
    print("---------------------------------------")

    for android_version in android_versions:
        Config.TARGET_ANDROID_VERSION = android_version
        # clone the apk repo if it doesn't exist
        if args.enable_git_clone:
            if Config.USE_CACHED_APKS:
                cached_repo = GitOp.clone_apk_source(android_version, release_type=Config.RELEASE_TYPE,
                                                             cached=True)
                Config.CACHED_SOURCE = cached_repo.working_tree_dir
            else:
                apk_repo = GitOp.clone_apk_source(android_version, args.arch, release_type=Config.RELEASE_TYPE)
                Config.APK_SOURCE = apk_repo.working_tree_dir
                overlay_repo = GitOp.clone_overlay_repo(android_version)
                if overlay_repo is not None:
                    Config.OVERLAY_SOURCE = overlay_repo.working_tree_dir
        if Config.OVERRIDE_RELEASE:
            Release.zip(package_list, android_version, args.arch, args.sign)
        if Config.RELEASE_TYPE and Config.ENVIRONMENT_TYPE == "production":
            GitOp.mark_a_release(android_version, Config.RELEASE_TYPE)
    if args.release:
        if args.update_website:
            website_repo = TestGit(GitStatics.website_repo_dir)
            if GitStatics.website_repo_url is not None:
                website_repo.clone_repo(repo_url=GitStatics.website_repo_url, fresh_clone=False, branch="main")
            if website_repo is not None:
                website_repo.update_changelog()

    t.taken("Total time taken by the program")

    print("End of the Program")


if __name__ == '__main__':
    main()



from niklibrary.helper.F import F
from niklibrary.helper.Statics import Statics
from NikGapps.helper.Package import Package
from niklibrary.compression.Modes import Modes


class CompOps:

    @staticmethod
    def get_compression_obj(file_name, compression_mode=Modes.ZIP):
        match compression_mode:
            case Modes.TAR_XZ:
                from niklibrary.compression.Tar import Tar
                return Tar(file_name)
            case _:
                from niklibrary.compression.Zip import Zip
                return Zip(file_name)

    @staticmethod
    def compress_package(pkg_zip_path, pkg: Package, compression_mode=Modes.DEFAULT):
        try:
            pkg_size = 0
            pkg_txt_path = pkg_zip_path.replace(compression_mode, "") + "_" + compression_mode[1:] + ".txt"
            cpkg = CompOps.get_compression_obj(pkg_zip_path, compression_mode=compression_mode)
            file_index = 1
            for x in pkg.file_dict:
                y = pkg.file_dict[x]
                file_index = file_index + 1
                pkg_size = pkg_size + Statics.get_file_bytes(x)
                cpkg.add_file(x, y)
            if pkg.clean_flash_only:
                cpkg.add_string("", "___etc___permissions/" + pkg.package_title + ".prop")
            pkg.pkg_size = pkg_size
            if pkg.priv_app_permissions_str is not None:
                cpkg.add_string(pkg.priv_app_permissions_str, "___etc___permissions/" + pkg.package_name + ".xml")
            cpkg.add_string(pkg.get_installer_script(str(pkg_size)), "installer.sh")
            cpkg.add_string(pkg.get_uninstaller_script(), "uninstaller.sh")
            cpkg.close()
            F.write_string_file(str(pkg_size), pkg_txt_path)
        except Exception as e:
            print(e)
            return False
        return True



import os
from dotenv import load_dotenv
from NikGapps.build.Build import Build
from NikGapps.config.NikGappsConfig import NikGappsConfig
from niklibrary.helper.F import F
from NikGapps.helper.Package import Package
from niklibrary.compression.Modes import Modes
from NikGapps.helper.compression.CompOps import CompOps
from NikGapps.helper.AppSet import AppSet
from NikGapps.helper import Config
from niklibrary.helper.P import P
from niklibrary.helper.T import T
from niklibrary.helper.SystemStat import SystemStat
from NikGapps.helper.Args import Args
from niklibrary.git.GitOp import GitOp
from niklibrary.git.GitlabManager import GitLabManager


def cache():
    args = Args()
    print("Start of the Program")
    SystemStat.show_stats()
    P.green("---------------------------------------")
    load_dotenv()
    Config.ENVIRONMENT_TYPE = os.getenv("ENVIRONMENT_TYPE") if os.getenv("ENVIRONMENT_TYPE") else "production"
    Config.RELEASE_TYPE = os.getenv("RELEASE_TYPE") if os.getenv("RELEASE_TYPE") else "stable"
    android_versions = [Config.TARGET_ANDROID_VERSION]
    Config.UPLOAD_FILES = args.upload
    if len(args.get_android_versions()) > 0:
        android_versions = args.get_android_versions()
    print("---------------------------------------")
    print("Android Versions to build: " + str(android_versions))
    print("---------------------------------------")
    for android_version in android_versions:
        arch = "arm64"
        url = f"{android_version}{('_' + arch if arch != 'arm64' else '')}_{Config.RELEASE_TYPE}"
        cached_url = url + "_cached"
        gitlab_manager = GitLabManager(private_token=os.getenv("GITLAB_TOKEN"))
        project = gitlab_manager.get_project(cached_url)
        gitattributes = """*.zip filter=lfs diff=lfs merge=lfs -text
        *.tar.xz filter=lfs diff=lfs merge=lfs -text"""
        if project:
            gitlab_manager.reset_repository(cached_url, sleep_for=10, gitattributes=gitattributes)
        else:
            project = gitlab_manager.create_repository(cached_url, provide_owner_access=True)
            gitlab_manager.create_and_commit_file(project_id=project.id, file_path=".gitattributes",
                                                  content=gitattributes)
        repo_cached = GitOp.clone_apk_url(url=cached_url, use_ssh_clone=True)
        apk_repo = GitOp.clone_apk_url(url=url)
        Config.APK_SOURCE = apk_repo.working_tree_dir
        if float(android_version) >= 12.1:
            overlay_repo = GitOp.clone_overlay_repo(android_version=str(android_version), fresh_clone=True)
            Config.OVERLAY_SOURCE = overlay_repo.working_tree_dir
        config_obj = NikGappsConfig(android_version)
        app_set_list = config_obj.package_manager.get_packages("all")
        config_obj.config_package_list = Build.build_from_directory(app_set_list, android_version)
        for appset in config_obj.config_package_list:
            appset: AppSet
            for pkg in appset.package_list:
                pkg: Package
                compression_modes = [Modes.ZIP, Modes.TAR_XZ]
                for mode in compression_modes:
                    t = T()
                    print(f"Compressing into {mode} {pkg.package_title} for {appset.title}")
                    pkg_zip_path = os.path.join(repo_cached.working_tree_dir, appset.title,
                                                f"{pkg.package_title}{mode}")
                    print("Done!") if CompOps.compress_package(pkg_zip_path, pkg, mode) else print("Failed!")
                    t.taken(f"Total time taken to process the {pkg.package_title}, compressing into {mode}")
                repo_cached.git_push(commit_message=f"Compressed {pkg.package_title} for {appset.title}",
                                     push_untracked_files=True, pull_first=True, post_buffer="1048576000")
        if Config.ENVIRONMENT_TYPE.__eq__("production") and F.dir_exists(repo_cached.working_tree_dir):
            F.remove_dir(repo_cached.working_tree_dir)


if __name__ == "__main__":
    cache()



import os
from pathlib import Path

from NikGapps.build.NikGappsOverlays import NikGappsOverlays
from NikGapps.helper.Args import Args
from niklibrary.helper.Cmd import Cmd
from niklibrary.helper.F import F
from niklibrary.helper.P import P
from niklibrary.helper.Statics import Statics
from niklibrary.git.Git import Git
from NikGapps.helper.overlay.Overlay import Overlay


def overlay_control():
    args = Args()
    android_versions = args.get_android_versions()

    for android_version in android_versions:
        repo_name = Statics.get_overlay_source_repo(android_version)
        repo_dir = Statics.get_overlay_source_directory(android_version)
        branch = "main"
        overlay_source_repo = Git(repo_dir)
        overlay_source_repo.clone_repo(repo_name, branch=branch)

        for overlay in NikGappsOverlays.get_overlay(android_version=android_version):
            overlay: Overlay
            overlay.build_apk_source(repo_dir)
            if overlay_source_repo.due_changes():
                print("Pushing due changes!")
                overlay_source_repo.git_push(commit_message=f"Updated {overlay.folder}!", push_untracked_files=True)

        if F.dir_exists(repo_dir):
            overlays_repo_name = Statics.get_overlay_repo(android_version)
            overlays_repo_dir = Statics.get_overlay_directory(android_version)
            overlay_repo = Git(overlays_repo_dir)
            overlay_repo.clone_repo(overlays_repo_name, branch="main")
            for folder in Path(repo_dir).iterdir():
                if str(folder).__contains__(".git") or str(folder).__contains__("README.md"):
                    continue
                cmd = Cmd()
                if not F.file_exists(os.path.join(str(folder), "apktool.yml")):
                    P.red(f"apktool.yml doesn't exist in {folder}")
                    continue
                overlay_path = cmd.build_overlay(folder_name=str(folder))
                if not overlay_path.__eq__(""):
                    print(f"{overlay_path} successfully built..")
                    print(
                        f"Copying to {os.path.join(overlays_repo_dir, str(Path(folder).name), f'{Path(folder).name}.apk')}")
                    F.copy_file(overlay_path, os.path.join(overlays_repo_dir,
                                                                str(Path(folder).name), f"{Path(folder).name}.apk"))
                    folder_to_remove = os.path.join(str(folder), "dist")
                    F.remove_dir(folder_to_remove)
                    folder_to_remove = os.path.join(str(folder), "build")
                    F.remove_dir(folder_to_remove)
                else:
                    print("Failed to build overlay")
            if overlay_repo.due_changes():
                print("Pushing due changes!")
                overlay_repo.git_push(commit_message="Updated Overlays!", push_untracked_files=True)
            else:
                print(f"{overlays_repo_dir} doesn't exist!")
        else:
            print(f"{repo_dir} doesn't exist!")


if __name__ == "__main__":
    overlay_control()



import concurrent.futures
import os
from multiprocessing import cpu_count
from pathlib import Path
from threading import Lock

from NikGapps.helper import Config
from NikGapps.helper.Package import Package
from niklibrary.helper.F import F
from niklibrary.helper.Cmd import Cmd
from NikGapps.helper.AppSet import AppSet
from niklibrary.helper.Statics import Statics
from niklibrary.helper.T import T


class Build:
    project_name = "NikGapps"

    # Just provide the package list, and it will pick them up from the directory and build them for you
    @staticmethod
    def build_from_directory(app_set_build_list, android_version, cached=False):
        if cached:
            return app_set_build_list
        building = T()
        source_directory = Config.APK_SOURCE
        print(f"Building from {source_directory}")
        cmd = Cmd()
        app_set_list = []
        max_workers = cpu_count()
        lock = Lock()

        def process_package(app_set_obj, pkg_obj):
            package_title = pkg_obj.package_title
            app_set_path = os.path.join(source_directory, app_set_obj.title)
            pkg_path = os.path.join(str(app_set_path), package_title)

            print(f"Setting up {app_set_obj.title}/{package_title}")

            file_dict = {}
            folder_dict = {}
            install_list = []
            delete_files_list = []
            delete_overlay_list = []
            package_name = None
            app_type = None
            primary_app_location = None

            if float(android_version) >= 12.1:
                overlay_directory = Config.OVERLAY_SOURCE
                overlay_dir = overlay_directory + Statics.dir_sep + f"{package_title}Overlay"
                if F.dir_exists(overlay_dir):
                    for file in Path(overlay_dir).rglob("*.apk"):
                        pkg_files_path = "overlay" + Statics.dir_sep + Path(file).name
                        install_list.append(pkg_files_path.replace("___", "/"))
                        file_dict_value = str(pkg_files_path.replace("___", "/")).replace("\\", "/")
                        value = str(file_dict_value).split("/")
                        file_dict[str(file)] = "___" + "___".join(value[:len(value) - 1]) + "/" + value[len(value) - 1]

            for pkg_files in Path(pkg_path).rglob("*"):
                if Path(pkg_files).is_dir() or str(pkg_files).__contains__(".git") \
                        or str(pkg_files).endswith(".gitattributes") or str(pkg_files).endswith("README.md"):
                    continue
                if str(pkg_files).endswith(Statics.DELETE_FILES_NAME):
                    for str_data in F.read_string_file(pkg_files):
                        delete_file = str_data.strip()
                        if delete_file not in delete_files_list:
                            delete_files_list.append(delete_file)
                    continue
                pkg_files_path = str(pkg_files)
                pkg_files_path = pkg_files_path[pkg_files_path.find("___") + 3:]
                if pkg_files_path.replace("\\", "/").__eq__(f"etc___permissions/{pkg_obj.package_name}.xml"):
                    continue
                if pkg_obj.package_name is not None and str(pkg_files_path).endswith(".apk") and not str(
                        pkg_files).__contains__("split_") and not str(pkg_files).__contains__("___m") \
                        and not str(pkg_files).__contains__("___overlay"):
                    primary_app_location = pkg_files.absolute()
                    package_name = cmd.get_package_details(primary_app_location, "name")
                    if str(primary_app_location).__contains__("___priv-app___"):
                        app_type = Statics.is_priv_app
                    elif str(primary_app_location).__contains__("___app___"):
                        app_type = Statics.is_system_app
                for folder in F.get_dir_list(pkg_files_path):
                    if folder.startswith("system") or folder.startswith("vendor") \
                            or folder.startswith("product") or folder.startswith("system_ext") \
                            or folder.startswith("overlay"):
                        continue
                    folder_dict[folder] = folder
                # We don't need this but for the sake of consistency
                if str(pkg_files_path).endswith("xml") or str(pkg_files_path).endswith("prop"):
                    F.convert_to_lf(str(pkg_files.absolute()))
                install_list.append(pkg_files_path.replace("___", "/"))
                file_dict_value = str(pkg_files_path.replace("___", "/")).replace("\\", "/")
                value = str(file_dict_value).split("/")
                file_dict[pkg_files.absolute()] = "___" + "___".join(value[:len(value) - 1]) + "/" + value[
                    len(value) - 1]

            if primary_app_location is not None:
                title = os.path.basename(primary_app_location)[:-4]
            else:
                title = package_title

            # Create a new Package object with the gathered data
            pkg = Package(title, package_name, app_type, package_title)
            pkg.install_list = install_list
            pkg.partition = pkg_obj.partition
            pkg.clean_flash_only = pkg_obj.clean_flash_only
            pkg.file_dict = file_dict
            pkg.folder_dict = folder_dict
            pkg.addon_index = pkg_obj.addon_index
            pkg.additional_installer_script = pkg_obj.additional_installer_script
            pkg.primary_app_location = primary_app_location

            if pkg.primary_app_location is not None and app_type == Statics.is_priv_app:
                permissions_list = cmd.get_white_list_permissions(primary_app_location)
                for perm in pkg_obj.priv_app_permissions:
                    permissions_list.append(perm)
                if permissions_list and not permissions_list[0].__contains__("Exception"):
                    pkg.priv_app_permissions_str = pkg.generate_priv_app_whitelist(app_set_obj.title, permissions_list,
                                                                                   android_version=android_version,
                                                                                   pkg_path=source_directory)
            pkg.delete_files_list = delete_files_list
            for delete_app in pkg_obj.delete_files_list:
                pkg.delete(delete_app)
            pkg.delete_overlay_list = delete_overlay_list
            pkg.validation_script = pkg_obj.validation_script
            pkg.overlay_list = pkg_obj.overlay_list

            return pkg

        with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
            app_set_futures = {}

            for app_set in app_set_build_list:
                futures = []
                for pkg_object in app_set.package_list:
                    future = executor.submit(process_package, app_set, pkg_object)
                    futures.append(future)
                app_set_futures[app_set.title] = futures

            for app_set_title, futures in app_set_futures.items():
                package_list = []
                for future in concurrent.futures.as_completed(futures):
                    package_list.append(future.result())

                if package_list:
                    with lock:
                        app_set_to_build = AppSet(app_set_title, package_list)
                        app_set_list.append(app_set_to_build)
        print(f"Total Time Taken: {building.taken()}")
        return app_set_list



import os
import xml.etree.ElementTree as eT

from niklibrary.helper.Statics import Statics


class Manifest:
    def __init__(self, package, android_version, target_package="android", priority="99",
                 is_static=True):
        self.package = package
        self.compileSdkVersion = Statics.get_android_sdk(android_version)
        self.compileSdkVersionCodename = android_version
        self.platformBuildVersionCode = Statics.get_android_sdk(android_version)
        self.platformBuildVersionName = android_version
        self.targetPackage = target_package
        self.priority = priority
        self.isStatic = str(is_static).lower()

    def to_xml(self):
        root = eT.Element('manifest',
                          {'xmlns:android': 'http://schemas.android.com/apk/res/android',
                           'package': self.package,
                           'android:compileSdkVersion': self.compileSdkVersion,
                           'android:compileSdkVersionCodename': self.compileSdkVersionCodename,
                           'platformBuildVersionCode': self.platformBuildVersionCode,
                           'platformBuildVersionName': self.platformBuildVersionName})

        eT.SubElement(root, 'application',
                      {'android:extractNativeLibs': 'true', 'android:hasCode': 'false'})
        eT.SubElement(root, 'overlay',
                      {'android:targetPackage': self.targetPackage,
                       'android:priority': self.priority, 'android:isStatic': self.isStatic})
        return eT.ElementTree(root)

    def indent(self, elem, level=0):
        i = "\n" + level * "  "
        if len(elem):
            if not elem.text or not elem.text.strip():
                elem.text = i + "  "
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
            for elem in elem:
                self.indent(elem, level + 1)
            if not elem.tail or not elem.tail.strip():
                elem.tail = i
        else:
            if level and (not elem.tail or not elem.tail.strip()):
                elem.tail = i

    def write(self, directory, filename):
        if not os.path.exists(directory):
            os.makedirs(directory)
        root = self.to_xml().getroot()
        self.indent(root)
        eT.ElementTree(root).write(os.path.join(directory, filename),
                                   encoding="utf-8", xml_declaration=False)



from NikGapps.config.NikGappsConfig import NikGappsConfig
from NikGapps.helper.Args import Args
from NikGapps.helper import Config


def config_upload():
    args = Args()
    android_versions = [Config.TARGET_ANDROID_VERSION]
    if len(args.get_android_versions()) > 0:
        android_versions = args.get_android_versions()

    for android_version in android_versions:
        if args.upload:
            Config.UPLOAD_FILES = True
            print("Uploading NikGapps Config for " + android_version)
            config = NikGappsConfig(android_version)
            config.upload_nikgapps_config()


if __name__ == "__main__":
    config_upload()



import os
from dotenv import load_dotenv

load_dotenv()

# The android version that we're targeting this application to run
TARGET_ANDROID_VERSION = 15

# Release type defines the release
# Possible values are [ 'canary', 'stable' ]
RELEASE_TYPE = os.environ.get('RELEASE_TYPE', 'stable')

# Environment type differentiates the experimental and stable features
# Possible values are [ 'production', 'development' ]
ENVIRONMENT_TYPE = os.environ.get('ENVIRONMENT_TYPE', 'dev')

# Possible Values are ['go', 'core', 'basic', 'omni', 'stock', 'full', 'addons', 'addonsets']
BUILD_PACKAGE_LIST = ['go', 'core', 'basic', 'omni', 'stock', 'full', 'addons', 'addonsets']

# Send the zip to device after creation, Possible values are True and False
SEND_ZIP_DEVICE = True if ENVIRONMENT_TYPE == "dev" and RELEASE_TYPE in ("dev", "beta") else False
SEND_ZIP_LOCATION = "/sdcard"

# This will allow the program to sign the zip
SIGN_ZIP = False

# When Fresh Build is True, the installer.sh will freshly build the zip (Comparatively Slower)
# When Fresh Build is False, the installer.sh picks up existing zip and builds gapps package (Faster)
FRESH_BUILD = True

# DEBUG_MODE will be helpful in printing more stuff so program can be debugged
DEBUG_MODE = True
if ENVIRONMENT_TYPE.__eq__("production"):
    DEBUG_MODE = False

# True if we want the files to upload as soon as they get created
UPLOAD_FILES = False

# True if we want to use cached apks
USE_CACHED_APKS = False

# Override the execution if we re-trigger the workflow
OVERRIDE_RELEASE = True

# Git Check enables controlled releases.
# If this is set to True, new release will only happen when there is a change in the source repo or apk is updated
GIT_CLONE_SOURCE = True
GIT_CHECK = True
GIT_PUSH = True

# Overlays, Gapps Apks and Cached Apks source
CACHED_SOURCE = None
APK_SOURCE = None
OVERLAY_SOURCE = None

# Enabling this will enable the feature of building NikGapps using config file
BUILD_CONFIG = True
BUILD_EXCLUSIVE = (RELEASE_TYPE.lower().__eq__("stable"))
EXCLUSIVE_FOLDER = "Elite"

PROJECT_MODE = "build"

# This will help fetch the files which requires root access such as overlay files
ADB_ROOT_ENABLED = False

TELEGRAM_BOT_TOKEN = os.environ.get('TELEGRAM_BOT_TOKEN')

TELEGRAM_CHAT_ID = os.environ.get('TELEGRAM_CHAT_ID')
NIKGAPPS_CHAT_ID = os.environ.get('NIKGAPPS_CHAT_ID')
MESSAGE_THREAD_ID = os.environ.get('MESSAGE_THREAD_ID')
ELITE_MESSAGE_THREAD_ID = os.environ.get('ELITE_MESSAGE_THREAD_ID')




import os
from niklibrary.helper.Statics import Statics
from NikGapps.helper.overlay.ApkMetaInfo import ApkMetaInfo
from NikGapps.helper.overlay.Manifest import Manifest


class Overlay:
    def __init__(self, apk_name, package_name, android_version, resources, priority="99", target_package="android"):
        self.folder = f"{apk_name}Overlay"
        self.apk_name = f"{self.folder}.apk"
        self.resources = resources
        self.manifest = Manifest(package=package_name, android_version=android_version, priority=priority,
                                 target_package=target_package)
        self.apkMetaInfo = ApkMetaInfo(apk_file_name=self.apk_name, android_version=android_version)

    def build_apk_source(self, source):
        build_dir = source + Statics.dir_sep + self.folder
        self.resources.write(os.path.join(build_dir, 'res/values'), 'config.xml')
        self.manifest.write(build_dir, 'AndroidManifest.xml')
        self.apkMetaInfo.write(build_dir, 'apktool.yml')



class PackageConstants:

    @staticmethod
    def get_package_clean_flash_rule(package_name):
        package_clean_flash_packages = ["com.google.android.googlequicksearchbox",
                                        "com.google.android.inputmethod.latin",
                                        "com.google.android.deskclock",
                                        "com.google.android.apps.googleassistant",
                                        "com.google.android.soundpicker",
                                        "com.google.android.projection.gearhead"]
        return package_name in package_clean_flash_packages

    @staticmethod
    def get_package_deletes(package_name):
        package_deletes = {
            "com.google.android.gms": ["PrebuiltGmsCoreQt", "PrebuiltGmsCoreRvc", "GmsCore"],
            "com.google.android.dialer": ["Dialer"],
            "com.google.android.contacts": ["Contacts"],
            "com.google.android.tts": ["PicoTts"],
            "com.google.android.inputmethod.latin": ["LatinIME"],
            "com.google.android.calendar": ["Calendar", "Etar", "SimpleCalendar"],
            "com.google.android.apps.messaging": ["RevengeMessages", "messaging", "Messaging", "QKSMS", "Mms"],
            "com.google.android.apps.photos": ["Gallery", "SimpleGallery", "Gallery2", "MotGallery", "MediaShortcuts",
                                               "SimpleGallery", "FineOSGallery", "GalleryX", "MiuiGallery",
                                               "SnapdragonGallery", "DotGallery", "Glimpse"],
            "com.google.android.keep": ["Notepad"],
            "com.google.android.apps.recorder": ["Recorder", "QtiSoundRecorder"],
            "com.google.android.gm": ["Email", "PrebuiltEmailGoogle"],
            "com.google.android.apps.wallpaper": ["Wallpapers", "ThemePicker"],
            "com.android.chrome": ["Bolt", "Browser", "Browser2", "BrowserIntl", "BrowserProviderProxy", "Chromium",
                                   "DuckDuckGo", "Fluxion", "Gello", "Jelly", "PA_Browser", "PABrowser", "YuBrowser",
                                   "BLUOpera", "BLUOperaPreinstall", "ViaBrowser", "Duckduckgo"],
            "com.google.android.youtube.music": ["SnapdragonMusic", "GooglePlayMusic", "Eleven", "CrDroidMusic"],
            "com.google.android.setupwizard": ["Provision", "SetupWizard", "LineageSetupWizard"],
            "com.google.android.pixel.setupwizard": ["Provision", "SetupWizard", "LineageSetupWizard"],
            "com.google.android.calculator": ["ExactCalculator", "MotoCalculator", "RevengeOSCalculator"],
            "com.google.android.apps.maps": ["Maps"],
            "com.google.android.apps.turbo": ["TurboPrebuilt"],
            "com.google.android.soundpicker": ["SoundPicker"],
            "com.google.android.storagemanager": ["StorageManager"],
            "com.google.android.documentsui": ["DocumentsUI"],
            "com.google.android.webview": ["webview"],
            "com.google.android.apps.restore": ["Seedvault"],
            "com.google.android.deskclock": ["DeskClock"],
            "org.lineageos.snap": ["GoogleCameraGo", "ScreenRecorder"],
            "com.google.android.as": ["DevicePersonalizationPrebuiltPixel2", "DevicePersonalizationPrebuiltPixel3",
                                      "DevicePersonalizationPrebuiltPixel4", "DevicePersonalizationPrebuiltPixel2020",
                                      "DevicePersonalizationPrebuiltPixel2021", "DevicePersonalizationPrebuiltPixel2021-v.U.14.playstore",
                                      "DevicePersonalizationPrebuiltPixel2021-aiai_20240329.00_RC16",
                                      "DevicePersonalizationPrebuiltPixel2021-U.32_V.7_playstore_aiai_20240725.00_RC08",
                                      "DevicePersonalizationPrebuiltPixel2022", "DevicePersonalizationPrebuiltPixel2022-v.U.14.playstore",
                                      "DevicePersonalizationPrebuiltPixel2022-aiai_20240329.00_RC16",
                                      "DevicePersonalizationPrebuiltPixel2022-U.32_V.7_playstore_aiai_20240725.00_RC08",
                                      "DevicePersonalizationPrebuiltPixel2023", "DevicePersonalizationPrebuiltPixel2023-v.U.14.playstore",
                                      "DevicePersonalizationPrebuiltPixel2023-aiai_20240329.00_RC16",
                                      "DevicePersonalizationPrebuiltPixel2023-U.32_V.7_playstore_aiai_20240725.00_RC08",
                                      "DevicePersonalizationPrebuiltPixel2024-aiai_20240329.00_RC16",
                                      "DevicePersonalizationPrebuiltPixelTablet2023-U.32_V.7_playstore_aiai_20240725.00_RC08",
                                      "DevicePersonalizationPrebuiltPixelTablet2024-U.32_V.7_playstore_aiai_20240725.00_RC08"],
            "com.google.android.as.oss": ["DeviceIntelligenceNetworkPrebuilt", "DeviceIntelligenceNetworkPrebuilt-v.U.14.playstore",
                                          "DeviceIntelligenceNetworkPrebuilt-astrea_20240329.00_RC02"],
            "com.google.android.aicore": ["AICorePrebuilt"],
            "com.google.android.apps.nexuslauncher": ["TrebuchetQuickStep", "Launcher3QuickStep", "ArrowLauncher",
                                                      "Lawnchair"],
            "com.android.systemui.plugin.globalactions.wallet": ["QuickAccessWallet"],
            "com.google.android.apps.youtube.music": ["SnapdragonMusic", "GooglePlayMusic", "Eleven", "CrDroidMusic"],
            "com.mixplorer.silver": ["MixPlorer"],
            "app.lawnchair": ["Lawnchair"]
        }
        return package_deletes.get(package_name, [])

    @staticmethod
    def get_package_script(package_name):
        package_scripts = {
            "ExtraFiles": """   script_text="<permissions>
                <!-- Shared library required on the device to get Google Dialer updates from
                     Play Store. This will be deprecated once Google Dialer play store
                     updates stop supporting pre-O devices. -->
                <library name=\\"com.google.android.dialer.support\\"
                  file=\\"$install_partition/framework/com.google.android.dialer.support.jar\\" />

                <!-- Starting from Android O and above, this system feature is required for
                     getting Google Dialer play store updates. -->
                <feature name=\\"com.google.android.apps.dialer.SUPPORTED\\" />
                <!-- Feature for Google Dialer Call Recording -->
                <feature name=\\"com.google.android.apps.dialer.call_recording_audio\\" />
            </permissions>"
               echo -e "$script_text" > $install_partition/etc/permissions/com.google.android.dialer.support.xml
               set_perm 0 0 0644 "$install_partition/etc/permissions/com.google.android.dialer.support.xml"
               update_prop "$install_partition/etc/permissions/com.google.android.dialer.support.xml" "install" "$propFilePath" "$package_title"
               if [ -f "$install_partition/etc/permissions/com.google.android.dialer.support.xml" ]; then
                 addToLog "- $install_partition/etc/permissions/com.google.android.dialer.support.xml Successfully Written!" "$package_title"
               fi
               script_text="<permissions>
                <library name=\\"com.google.android.maps\\"
                        file=\\"$install_partition/framework/com.google.android.maps.jar\\" />
            </permissions>"
               echo -e "$script_text" > $install_partition/etc/permissions/com.google.android.maps.xml
               set_perm 0 0 0644 "$install_partition/etc/permissions/com.google.android.maps.xml"
               update_prop "$install_partition/etc/permissions/com.google.android.maps.xml" "install" "$propFilePath" "$package_title"
               if [ -f "$install_partition/etc/permissions/com.google.android.maps.xml" ]; then
                 addToLog "- $install_partition/etc/permissions/com.google.android.maps.xml Successfully Written!" "$package_title"
               fi
               script_text="<permissions>
            <library name=\\"com.google.widevine.software.drm\\"
            file=\\"/system/product/framework/com.google.widevine.software.drm.jar\\"/>
            </permissions>"
               echo -e "$script_text" > $install_partition/etc/permissions/com.google.widevine.software.drm.xml
               set_perm 0 0 0644 "$install_partition/etc/permissions/com.google.widevine.software.drm.xml"
               update_prop "$install_partition/etc/permissions/com.google.widevine.software.drm.xml" "install" "$propFilePath" "$package_title"
               if [ -f "$install_partition/etc/permissions/com.google.widevine.software.drm.xml" ]; then
                 addToLog "- $install_partition/etc/permissions/com.google.widevine.software.drm.xml Successfully Written!" "$package_title"
               fi
               script_text="<permissions>
            <library name=\\"com.google.android.media.effects\\"
            file=\\"$install_partition/framework/com.google.android.media.effects.jar\\" />

            </permissions>"
               echo -e "$script_text" > $install_partition/etc/permissions/com.google.android.media.effects.xml
               set_perm 0 0 0644 "$install_partition/etc/permissions/com.google.android.media.effects.xml"
               update_prop "$install_partition/etc/permissions/com.google.android.media.effects.xml" "install" "$propFilePath" "$package_title"
               if [ -f "$install_partition/etc/permissions/com.google.android.media.effects.xml" ]; then
                 addToLog "- $install_partition/etc/permissions/com.google.android.media.effects.xml Successfully Written!" "$package_title"
               fi
        """,
            "ExtraFilesGo": """   script_text="<permissions>
        <!-- Shared library required on the device to get Google Dialer updates from
             Play Store. This will be deprecated once Google Dialer play store
             updates stop supporting pre-O devices. -->
        <library name=\\"com.google.android.dialer.support\\"
          file=\\"$install_partition/framework/com.google.android.dialer.support.jar\\" />

        <!-- Starting from Android O and above, this system feature is required for
             getting Google Dialer play store updates. -->
        <feature name=\\"com.google.android.apps.dialer.SUPPORTED\\" />
        <!-- Feature for Google Dialer Call Recording -->
        <feature name=\\"com.google.android.apps.dialer.call_recording_audio\\" />
    </permissions>"
       echo -e "$script_text" > $install_partition/etc/permissions/com.google.android.dialer.support.xml
       set_perm 0 0 0644 "$install_partition/etc/permissions/com.google.android.dialer.support.xml"
       update_prop "$install_partition/etc/permissions/com.google.android.dialer.support.xml" "install" "$propFilePath" "$package_title"
       if [ -f "$install_partition/etc/permissions/com.google.android.dialer.support.xml" ]; then
         addToLog "- $install_partition/etc/permissions/com.google.android.dialer.support.xml Successfully Written!" "$package_title"
       fi
       script_text="<permissions>
        <library name=\\"com.google.android.maps\\"
                file=\\"$install_partition/framework/com.google.android.maps.jar\\" />
    </permissions>"
       echo -e "$script_text" > $install_partition/etc/permissions/com.google.android.maps.xml
       set_perm 0 0 0644 "$install_partition/etc/permissions/com.google.android.maps.xml"
       update_prop "$install_partition/etc/permissions/com.google.android.maps.xml" "install" "$propFilePath" "$package_title"
       if [ -f "$install_partition/etc/permissions/com.google.android.maps.xml" ]; then
         addToLog "- $install_partition/etc/permissions/com.google.android.maps.xml Successfully Written!" "$package_title"
       fi
       script_text="<permissions>
    <library name=\\"com.google.widevine.software.drm\\"
    file=\\"/system/product/framework/com.google.widevine.software.drm.jar\\"/>
    </permissions>"
       echo -e "$script_text" > $install_partition/etc/permissions/com.google.widevine.software.drm.xml
       set_perm 0 0 0644 "$install_partition/etc/permissions/com.google.widevine.software.drm.xml"
       update_prop "$install_partition/etc/permissions/com.google.widevine.software.drm.xml" "install" "$propFilePath" "$package_title"
       if [ -f "$install_partition/etc/permissions/com.google.widevine.software.drm.xml" ]; then
         addToLog "- $install_partition/etc/permissions/com.google.widevine.software.drm.xml Successfully Written!" "$package_title"
       fi
       script_text="<permissions>
    <library name=\\"com.google.android.media.effects\\"
    file=\\"$install_partition/framework/com.google.android.media.effects.jar\\" />

    </permissions>"
       echo -e "$script_text" > $install_partition/etc/permissions/com.google.android.media.effects.xml
       set_perm 0 0 0644 "$install_partition/etc/permissions/com.google.android.media.effects.xml"
       update_prop "$install_partition/etc/permissions/com.google.android.media.effects.xml" "install" "$propFilePath" "$package_title"
       if [ -f "$install_partition/etc/permissions/com.google.android.media.effects.xml" ]; then
         addToLog "- $install_partition/etc/permissions/com.google.android.media.effects.xml Successfully Written!" "$package_title"
       fi
        """,
            "GmsCore": """
    gms_optimization=$(ReadConfigValue "GmsOptimization" "$nikgapps_config_file_name")
    [ -z "$gms_optimization" ] && gms_optimization=0
    if [ "$gms_optimization" = "1" ]; then
        sed -i '/allow-in-power-save package=\"com.google.android.gms\"/d' $install_partition/etc/permissions/*.xml
        sed -i '/allow-in-data-usage-save package=\"com.google.android.gms\"/d' $install_partition/etc/permissions/*.xml
        sed -i '/allow-unthrottled-location package=\"com.google.android.gms\"/d' $install_partition/etc/permissions/*.xml
        sed -i '/allow-ignore-location-settings package=\"com.google.android.gms\"/d' $install_partition/etc/permissions/*.xml
        addToLog \"- Battery Optimization Done in $install_partition/etc/permissions/*.xml!\" "$package_title"
        sed -i '/allow-in-power-save package=\"com.google.android.gms\"/d' $install_partition/etc/sysconfig/*.xml
        sed -i '/allow-in-data-usage-save package=\"com.google.android.gms\"/d' $install_partition/etc/sysconfig/*.xml
        sed -i '/allow-unthrottled-location package=\"com.google.android.gms\"/d' $install_partition/etc/sysconfig/*.xml
        sed -i '/allow-ignore-location-settings package=\"com.google.android.gms\"/d' $install_partition/etc/sysconfig/*.xml
        addToLog \"- Battery Optimization Done in $install_partition/etc/sysconfig/*.xml!\" "$package_title"
    else
        addToLog "- Battery Optimization not Enabled" "$package_title"
    fi
        """,
            "SetupWizard": """
       set_prop "setupwizard.feature.baseline_setupwizard_enabled" "true" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "ro.setupwizard.enterprise_mode" "1" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "ro.setupwizard.rotation_locked" "true" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "setupwizard.enable_assist_gesture_training" "true" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "setupwizard.theme" "glif_v3_light" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "setupwizard.feature.skip_button_use_mobile_data.carrier1839" "true" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "setupwizard.feature.show_pai_screen_in_main_flow.carrier1839" "false" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "setupwizard.feature.show_pixel_tos" "false" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "setupwizard.feature.show_digital_warranty" "true" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "ro.setupwizard.esim_cid_ignore" "00000001" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "ro.setupwizard.setupwizard.feature.show_support_link_in_deferred_setup" "false" "$product/etc/build.prop" "$package_title"
       set_prop "setupwizard.feature.enable_wifi_tracker" "true" "$product/etc/build.prop" "$package_title"
       set_prop "setupwizard.feature.day_night_mode_enabled" "true" "$product/etc/build.prop" "$package_title"
       set_prop "setupwizard.feature.portal_notification" "true" "$product/etc/build.prop" "$package_title"
       set_prop "setupwizard.feature.lifecycle_refactoring" "true" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "setupwizard.feature.notification_refactoring" "true" "$product/etc/build.prop" "$package_title"
        """,
            "Velvet": """
       set_prop "ro.opa.eligible_device" "true" "$product/etc/build.prop" "$propFilePath" "$package_title"
        """,
            "GBoard": """
       set_prop "ro.com.google.ime.theme_id" "5" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "ro.com.google.ime.kb_pad_port_b" "8" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "ro.com.google.ime.height_ratio" "1.025" "$product/etc/build.prop" "$propFilePath" "$package_title"
       set_prop "ro.com.google.ime.system_lm_dir" "$install_partition/usr/share/ime/google/d3_lms" "$product/etc/build.prop" "$propFilePath" "$package_title"
        """
        }
        return package_scripts.get(package_name, None)


import concurrent.futures
import concurrent.futures
import os
import threading
from multiprocessing import cpu_count
from threading import Lock

from .CompOps import CompOps
from niklibrary.compression.Modes import Modes
from niklibrary.compression.Zip import Zip
from .. import Config
from ..Assets import Assets
from niklibrary.helper.Cmd import Cmd
from niklibrary.helper.F import F
from niklibrary.helper.P import P
from niklibrary.helper.Statics import Statics
from niklibrary.helper.T import T
from niklibrary.web.TelegramApi import TelegramApi
from ...config.NikGappsConfig import NikGappsConfig


class Export:
    def __init__(self, file_name, sign=False):
        self.file_name = file_name[:-4] + "-signed.zip" if sign else file_name[:-4] + "-unofficial.zip"
        self.z = Zip(self.file_name, sign=sign, private_key_path=Assets.private_key_pem)

    def zip(self, config_obj: NikGappsConfig, telegram: TelegramApi = TelegramApi(None, None),
            compression_mode=Modes.DEFAULT, send_zip_device=Config.SEND_ZIP_DEVICE):

        build_zip = T()
        app_set_list = config_obj.config_package_list
        max_workers = cpu_count()
        lock = Lock()
        total_packages = sum(len(app_set.package_list) for app_set in app_set_list)
        completed_packages = 0
        file_sizes = ""
        zip_execution_status = False
        telegram.message(f"- Gapps is building {0:.2f}% ({completed_packages}/{total_packages})")

        def compress_and_add_file(appset, pakg, compressionmode):
            nonlocal completed_packages
            nonlocal file_sizes
            thread_id = threading.get_ident()

            pkg_zip_path = (
                    Statics.get_temp_packages_directory(config_obj.android_version, arch=config_obj.arch)
                    + Statics.dir_sep + "Packages" + Statics.dir_sep + str(pakg.package_title) + compressionmode
            )
            pkg_txt_path = pkg_zip_path.replace(compression_mode, "") + "_" + compression_mode[1:] + ".txt"

            if Config.USE_CACHED_APKS:
                print(f"Using cached package: {os.path.basename(pkg_zip_path)}")
                cached_pkg_zip_path = os.path.join(Config.CACHED_SOURCE, appset.title,
                                                   f"{pakg.package_title}{compression_mode}")
                if F.file_exists(cached_pkg_zip_path):
                    pkg_zip_path = cached_pkg_zip_path
                    pkg_txt_path = pkg_zip_path.replace(compression_mode, "") + "_" + compression_mode[
                                                                                      1:] + ".txt"
                else:
                    print(cached_pkg_zip_path + " doesn't exist!")
            else:
                CompOps.compress_package(pkg_zip_path, pakg, compression_mode)

            for size_on_file in F.read_string_file(pkg_txt_path):
                pkg_size = size_on_file
                pakg.pkg_size = pkg_size

            with lock:
                self.z.add_file(pkg_zip_path, f"AppSet/{appset.title}/{pakg.package_title}{compressionmode}")
                file_sizes = file_sizes + str(pakg.package_title) + "=" + str(pakg.pkg_size) + "\n"
                completed_packages += 1
                progress = (completed_packages / total_packages) * 100
                telegram.message(f"- Gapps is building {progress:.2f}% ({completed_packages}/{total_packages})",
                                 replace_last_message=True)
                P.blue(f"- Gapps is building {progress:.2f}% ({completed_packages}/{total_packages}): "
                       f"{appset.title}/{pakg.package_title} (Thread {thread_id})")

        with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
            futures = []
            for app_set in app_set_list:
                for pkg in app_set.package_list:
                    futures.append(executor.submit(compress_and_add_file, app_set, pkg, compression_mode))

        self.z.add_string(self.get_installer_script(total_packages, app_set_list, compression_mode),
                          "common/install.sh")
        self.z.add_string("#MAGISK", Statics.meta_inf_dir + "updater-script")
        self.z.add_file(Assets.magisk_update_binary, Statics.meta_inf_dir + "update-binary")
        self.z.add_string(config_obj.get_nikgapps_config(), "afzc/nikgapps.config")
        debloater_config_lines = ""
        for line in Assets.get_string_resource(Assets.debloater_config):
            debloater_config_lines += line
        for line in config_obj.debloater_list:
            debloater_config_lines += line + "\n"
        self.z.add_string(debloater_config_lines, "afzc/debloater.config")
        self.z.add_file(Assets.changelog, "changelog.yaml")
        self.z.add_file(Assets.addon_path, "common/addon.sh")
        self.z.add_file(Assets.header_path, "common/header.sh")
        self.z.add_file(Assets.functions_path, "common/functions.sh")
        self.z.add_string(file_sizes, "common/file_size.txt")
        self.z.add_file(Assets.nikgapps_functions, "common/nikgapps_functions.sh")
        self.z.add_file(Assets.mount_path, "common/mount.sh")
        self.z.add_file(Assets.mtg_mount_path, "common/mtg_mount.sh")
        self.z.add_file(Assets.unmount_path, "common/unmount.sh")
        self.z.add_string(os.path.basename(os.path.splitext(self.file_name)[0]), "zip_name.txt")
        self.z.add_string(f"Created by {config_obj.creator}".center(38, ' '), "creator.txt")
        self.z.add_string(self.get_customize_sh(self.file_name), "customize.sh")
        self.z.add_file(Assets.module_path, "module.prop")
        self.z.add_file(Assets.busybox, "busybox")
        self.z.close()
        zip_execution_status = True
        P.green('The zip ' + self.file_name + ' is created successfully!')
        time_taken = build_zip.taken("Total time taken to build the zip")
        telegram.message("- Completed in: " + T.format_time(round(time_taken)))
        if send_zip_device:
            send_zip_device_time = T()
            cmd = Cmd()
            device_path = f"{Config.SEND_ZIP_LOCATION}/Afzc-" + str(
                config_obj.android_version) + "/" + T.get_current_time() + "/" + os.path.basename(self.file_name)
            message = f"Sending {os.path.basename(self.file_name)} to device at: " + device_path
            print(message)
            cmd.push_package(self.file_name, device_path)
            send_zip_device_time.taken("Total time taken to send the zip to device")
        return self.file_name, zip_execution_status

    @staticmethod
    def get_installer_script(total_packages, app_set_list, compression_mode=Modes.DEFAULT):
        delem = ","
        installer_script_path_string = "#!/sbin/sh\n"
        installer_script_path_string += "# Shell Script EDIFY Replacement\n\n"
        core_app_sets = []
        other_app_sets = []
        for app_set in app_set_list:
            if app_set.title in ['Core', 'CoreGo']:
                core_app_sets.append(app_set)
            else:
                other_app_sets.append(app_set)
        sorted_other_app_sets = sorted(other_app_sets,
                                       key=lambda app__set: sum(int(pakg.pkg_size) for pakg in app__set.package_list),
                                       reverse=True)
        sorted_app_sets = core_app_sets + sorted_other_app_sets
        progress_max = 0.9
        progress_per_package = 0
        if total_packages > 0:
            progress_per_package = round(progress_max / total_packages, 2)
        install_progress = 0
        installer_script_path_string += f"ProgressBarValues=\"\n"
        for app_set in sorted_app_sets:
            sorted_packages = sorted(app_set.package_list, key=lambda pakg: pakg.pkg_size, reverse=True)
            for pkg in sorted_packages:
                install_progress += progress_per_package
                if install_progress > 1.0:
                    install_progress = 1.0
                installer_script_path_string += f"{pkg.package_title}={str(round(install_progress, 2))}\n"
        installer_script_path_string += "\"\n\n"

        for app_set in sorted_app_sets:
            sorted_packages = sorted(app_set.package_list, key=lambda pakg: pakg.pkg_size, reverse=True)
            installer_script_path_string += f"{app_set.title}=\"\n"
            for pkg in sorted_packages:
                installer_script_path_string += f"{pkg.package_title}{delem}{pkg.pkg_size}{delem}{pkg.partition}\n"
            installer_script_path_string += "\"\n\n"

        for app_set in sorted_app_sets:
            installer_script_path_string += "install_app_set \"" + \
                                            app_set.title + "\" \"$" + \
                                            app_set.title + "\" \"" + \
                                            compression_mode + "\" \n"

        installer_script_path_string += "\nset_progress 1.00" + "\n\n"
        installer_script_path_string += "exit_install" + "\n\n"
        return installer_script_path_string

    @staticmethod
    def get_customize_sh(file_name):
        customize_path_string = "actual_file_name=" + os.path.basename(os.path.splitext(file_name)[0]) + "\n"
        lines = Assets.get_string_resource(Assets.customize_path)
        for line in lines:
            customize_path_string += line
        return customize_path_string



import argparse
import os
from dotenv import load_dotenv
from NikGapps.helper import Config
from niklibrary.helper.SystemStat import SystemStat
from niklibrary.git.GitlabManager import GitLabManager


def copy_repos():
    parser = argparse.ArgumentParser(description='Copy Repos Parser')
    parser.add_argument('-S', '--source', default="", help='Source Repository')
    parser.add_argument('-D', '--destination', default="", help='Destination Repository')
    parser.add_argument('-A', '--android_version', default="", help='Android version')
    args = parser.parse_args()
    android_versions = str(args.android_version).replace("'", "").split(',')
    if not android_versions or (len(android_versions) > 0 and android_versions[0] == ""):
        android_versions = [Config.TARGET_ANDROID_VERSION]
    print("Start of the Program")
    SystemStat.show_stats()
    load_dotenv()
    Config.ENVIRONMENT_TYPE = os.getenv("ENVIRONMENT_TYPE") if os.getenv("ENVIRONMENT_TYPE") else "production"
    Config.RELEASE_TYPE = os.getenv("RELEASE_TYPE") if os.getenv("RELEASE_TYPE") else "stable"

    # Config.UPLOAD_FILES = args.upload
    print("---------------------------------------")
    print("Android Versions to build: " + str(android_versions))
    print("---------------------------------------")

    for android_version in android_versions:
        gitlab_manager = GitLabManager(private_token=os.getenv("GITLAB_TOKEN"))
        gitlab_manager.copy_repository(f"{android_version}_{args.source}", f"{android_version}_{args.destination}",
                                       override_target=True)
        # gitlab_manager.copy_repository(f"{android_version}_stable", f"{android_version}_ondemand", override_target=True)
        # gitlab_manager.copy_repository(f"{android_version}_stable_cached", f"{android_version}_ondemand_cached",
        #                                override_target=True)


if __name__ == "__main__":
    copy_repos()
